# encoding: utf-8
from __future__ import absolute_import, unicode_literals

import logging

from django.http import HttpResponse
from django.conf import settings
from django.utils.encoding import force_text
from rest_framework.response import Response

from . import utility
from .views.rest_framework import View as ViewBase
from .err_code import ErrCode
from .exceptions import CustomError
from .logger import CALLER_KEY


class APIView(ViewBase):
    ERROR_CODE_STATUS_CODE = 400
    PARAM_ERRORS_STATUS_CODE = 400
    SUCCESS_WITH_CODE = True
    logger = logging.getLogger('apiview')

    def get_view_name(self):
        if hasattr(self, 'name'):
            return self.name
        return super(APIView, self).get_view_name()

    def initial(self, request, *args, **kwargs):
        self.logger.info(
            "m=%s g=%s p=%s w=%s u=%s",
            request.META,
            request.query_params,
            getattr(request, 'body', None),
            getattr(self, 'ws_data', None),
            request.user,
            extra={CALLER_KEY: self.get_context}
        )
        super(APIView, self).initial(request, *args, **kwargs)

    def format_res_data(self, context, status_code=200):
        if self.SUCCESS_WITH_CODE:
            if not isinstance(context, dict) or 'code' not in context:
                context = self.get_default_context(data=context)

        return Response(utility.format_res_data(context), status=status_code)

    def check_api_permissions(self, request, *args, **kwargs):
        pass

    def view(self, request, *args, **kwargs):
        self.check_api_permissions(request, *args, **kwargs)
        context = self.get_context(request, *args, **kwargs)
        if isinstance(context, HttpResponse):
            response = context
        else:
            response = self.format_res_data(context)
        return response

    def get(self, request, *args, **kwargs):
        return self.view(request, *args, **kwargs)

    post = ws = get

    @staticmethod
    def get_default_context(**kwargs):
        return ErrCode.SUCCESS.get_res_dict(**kwargs)

    def get_context(self, request, *args, **kwargs):
        raise NotImplementedError

    def handle_param_errors(self, exc):
        request = self.request
        self.logger.info(
            "param_errors m=%s g=%s p=%s w=%s u=%s",
            request.META,
            request.query_params,
            getattr(request, 'body', None),
            getattr(self, 'ws_data', None),
            request.user,
            extra={CALLER_KEY: self.get_context}
        )
        context = ErrCode.ERR_COMMON_BAD_PARAM.get_res_dict()
        if getattr(settings, 'APIVIEW_SHOWPARAM_INFO', settings.DEBUG):
            if hasattr(exc, 'error_dict_obj'):
                context['errors'] = exc.error_dict_obj
                context['desc'] = exc.error_dict_obj.as_text()
            else:
                context['desc'] = force_text(exc)
        return self.format_res_data(context, status_code=self.PARAM_ERRORS_STATUS_CODE)

    def handle_exception(self, exc):
        if isinstance(exc, CustomError):
            return self.format_res_data(exc.get_res_dict(), status_code=self.ERROR_CODE_STATUS_CODE)

        if settings.DEBUG:
            raise exc

        utility.reportExceptionByMail("500")
        return self.format_res_data(ErrCode.ERR_SYS_ERROR.get_res_dict(), status_code=self.ERROR_CODE_STATUS_CODE)



# encoding: utf-8
from __future__ import absolute_import, unicode_literals

import json
import six
from hashlib import md5

from io import BytesIO
from StringIO import StringIO
from importlib import import_module

from django.conf import settings
from django import http, forms
from django.db import models, transaction
from django.core.urlresolvers import resolve
from django.utils.translation import ugettext_lazy as _
from django.utils import datastructures
from django.utils.functional import SimpleLazyObject, cached_property

TEST_PARAM = '_test_case_'
# md5('testing-gg-5566').hexdigest()
TEST_KEY = '8F7CE16E134926D721AB3D109714DF42'
TEST_REQUEST_PARAM = '_test_case_request_'


class JSONFormField(forms.CharField):

    def to_python(self, value):
        if value in self.empty_values:
            return None
        return json.loads(value)

    def prepare_value(self, value):
        value = self.to_python(value)
        if value is not None:
            value = json.dumps(value, indent=4)
        return value


class JSONField(models.TextField):
    def to_python(self, value):
        if isinstance(value, six.string_types):
            return json.loads(value)
        return value

    def value_to_string(self, obj):
        val = self._get_val_from_obj(obj)
        val = self.to_python(val)
        return json.dumps(val)

    def get_prep_value(self, value):
        if not value:
            return ''
        return json.dumps(self.to_python(value))

    def formfield(self, **kwargs):
        defaults = {'form_class': JSONFormField}
        defaults.update(**kwargs)
        return super(JSONField, self).formfield(**defaults)


class TestCase(models.Model):
    path = models.CharField(_('path'), max_length=100, db_index=True)
    request_md5 = models.CharField(_('md5'), max_length=32, db_index=True, editable=False)
    request_data = JSONField(_('request data'))
    response_data = JSONField(_('response data'), blank=True)
    response_schema = JSONField(_('response schema'), blank=True)

    def __unicode__(self):
        return "<%s:%s>" % (self.__class__.__name__, self.request_md5)

    def as_request(self):
        assert self.request_data, 'Could not construct an empty TestCase object'
        request = http.HttpRequest()
        data = self.request_data_dict
        request.path = data['path']
        request.method = data['method']
        request.path_info = data['path_info']
        request._body = data['body']
        request.META = data['headers']
        request._encoding = data['encoding']
        request._stream = StringIO()
        request._read_started = False
        request._post_parse_error = False
        request.resolver_match = None

        request._load_post_and_files()
        query_string = '%s&%s=1' % (data['query_string'], TEST_REQUEST_PARAM)
        request.GET = http.QueryDict(query_string, encoding=request.encoding)
        request.POST = getattr(request, '_post')
        request.FILES = getattr(request, '_files')
        request.COOKIES = http.parse_cookie(request.META.get('HTTP_COOKIE', b''))
        request.REQUEST = datastructures.MergeDict(request.POST, request.GET)

        # extra attributes added by middlewares
        from django.contrib.auth.middleware import get_user
        engine = import_module(settings.SESSION_ENGINE)
        request.session = engine.SessionStore(request.COOKIES.get(settings.SESSION_COOKIE_NAME, None))
        request.user = SimpleLazyObject(lambda: get_user(request))
        return request

    def as_response(self):
        assert self.response_data, 'Could not construct an empty TestCase object'
        response = http.HttpResponse()
        data = self.response_data_dict
        response._charset = data['encoding']
        response.status_code = data['status_code']
        response._headers = data['headers']

        response.content = data['content']
        return response

    @property
    def request_data_dict(self):
        if not self.request_data:
            return None
        elif isinstance(self.request_data, six.string_types):
            self.request_data = json.loads(self.request_data)
        return self.request_data

    @property
    def response_data_dict(self):
        if not self.response_data:
            return None
        elif isinstance(self.response_data, six.string_types):
            self.response_data = json.loads(self.response_data)
        return self.response_data

    @property
    def response_schema_dict(self):
        if not self.response_schema:
            return None
        elif isinstance(self.response_schema, six.string_types):
            self.response_schema = json.loads(self.response_schema)
        return self.response_schema

    def load_request(self, request):
        path = request.path
        query_string = request.META.get('QUERY_STRING', '')
        try:
            body = request.body
        except Exception:
            body = request._body = request.read()
            request._stream = BytesIO(request._body)

        request_headers = {}
        for name, value in request.META.items():
            if name.startswith('HTTP_'):
                request_headers[name] = value
            elif name in ('CONTENT_TYPE', ):
                request_headers[name] = value

        request_data = {
            'path': path,
            'encoding': request.encoding or settings.DEFAULT_CHARSET,
            'path_info': request.path_info,
            'method': request.method,
            'query_string': query_string,
            'body': body,
            'headers': request_headers,
        }

        self.path = path
        self.request_data = request_data
        self.request_md5 = md5(json.dumps(self.request_data)).hexdigest()
        try:
            case = TestCase.objects.get(
                    path=self.path,
                    request_md5=self.request_md5)
            self.__dict__ = case.__dict__
        except TestCase.DoesNotExist:
            pass

    def load_response(self, response, confirm=False):
        if self.response_schema and not confirm:
            return

        if hasattr(response, 'render') and callable(response.render):
            response.render()

        content_type = response['Content-Type']
        if 'json' in content_type:
            data_type = 'json'
        elif 'html' in content_type:
            data_type = 'html'
        elif 'xml' in content_type:
            data_type = 'xml'
        else:
            data_type = 'binary'

        response_data = {
            'status_code': response.status_code,
            'type': data_type,
            'encoding': response._charset,
            'headers': response._headers,
            'content': response.content,
            }

        self.response_data = response_data

        schema_method = getattr(self, 'schema_' + data_type, None)
        if schema_method:
            schema = schema_method(response.content)
            self.response_schema = schema
        else:
            schema = None

    def test(self):
        assert self.request_data, 'Could not construct an empty TestCase object'

        match = resolve(self.path)
        with transaction.commit_manually():
            try:
                response = match.func(self.as_request(), *match.args, **match.kwargs)
                return response
            finally:
                transaction.rollback()

    @cached_property
    def test_response(self):
        return self.test()

    @cached_property
    def errors(self):
        return list(self.gen_errors())

    def reset_errors(self):
        self.errors = list(self.gen_errors())

    def validate(self):
        return not bool(next(self.gen_errors(), False))

    def gen_errors(self):
        if not self.response_schema or not self.response_data:
            return

        response = self.test_response
        if hasattr(response, 'render') and callable(response.render):
            response.render()

        data = self.response_data_dict
        validate_method = getattr(self, 'validate_' + data['type'])
        for err in validate_method(response.content):
            yield err

    def schema_json(self, content):
        from .dtd.jsonschema import json2schema, gen_errors, flex_schema_by_errs
        data = json.loads(content)
        if self.response_schema:
            schema = self.response_schema_dict
            errors = list(gen_errors(data, schema))
            flex_schema_by_errs(schema, errors)
        else:
            schema = json2schema(data)
        return schema

    def validate_json(self, content):
        from .dtd.jsonschema import gen_errors
        for err in gen_errors(json.loads(content), json.loads(self.response_schema)):
            yield err



#! /usr/bin/env python
# encoding: utf-8
from __future__ import unicode_literals, absolute_import

import unittest

from .jsonschema import json2schema, gen_errors, flex_schema_by_errs


class TestJSONSchema(unittest.TestCase):
    def test_json2schema(self):
        self.assertEqual(json2schema([])['type'], 'array')
        self.assertEqual(json2schema({})['type'], 'object')
        self.assertEqual(json2schema({'a': 1.1})['properties']['a']['type'], 'number')
        self.assertEqual(json2schema({'a': 1})['properties']['a']['type'], 'integer')
        self.assertEqual(json2schema({'a': ''})['properties']['a']['type'], 'string')
        self.assertEqual(json2schema({'a': []})['properties']['a']['type'], 'array')
        self.assertEqual(json2schema({'a': {}})['properties']['a']['type'], 'object')

    def test_flex_schema_by_errs(self):
        data = {
            'string': 'value',
            'int': 1,
            'num': 1.0,
            'array': [1, 2, 3, 4],
            'object': {
                'sub_object': {
                    'sub_string': 'value',
                    'sub_int': 1,
                    'sub_num': 1.0,
                    'sub_array': [{'in_array': 'string'}, {'in_array': 1}, ],
                }},
            }

        schema = json2schema(data)

        # flex value
        data['int'] = 2
        self.assertEqual(next(gen_errors(data, schema)).validator, 'value')
        flex_schema_by_errs(schema, list(gen_errors(data, schema)))
        self.assertFalse(list(gen_errors(data, schema)))

        # flex integer - number
        data['int'] = 2.0
        self.assertEqual(next(gen_errors(data, schema)).validator, 'type')
        flex_schema_by_errs(schema, list(gen_errors(data, schema)))
        self.assertFalse(list(gen_errors(data, schema)))

        # the path generation
        data['object']['sub_object']['sub_int'] = 2
        self.assertEqual(next(gen_errors(data, schema)).validator, 'value')
        flex_schema_by_errs(schema, list(gen_errors(data, schema)))
        self.assertFalse(list(gen_errors(data, schema)))

        # flex minItems and value
        data['array'] = [1, 2]
        self.assertEqual(len(list(gen_errors(data, schema))), 2)
        flex_schema_by_errs(schema, list(gen_errors(data, schema)))
        self.assertFalse(list(gen_errors(data, schema)))

        # flex maxItems
        data['array'] = [1, 2, 3, 4] * 2
        self.assertEqual(next(gen_errors(data, schema)).validator, 'maxItems')
        flex_schema_by_errs(schema, list(gen_errors(data, schema)))
        self.assertFalse(list(gen_errors(data, schema)))

        # flex anyOf
        data['array'] = [1, 2, '5']
        self.assertEqual(next(gen_errors(data, schema)).validator, 'anyOf')
        flex_schema_by_errs(schema, list(gen_errors(data, schema)))
        self.assertFalse(list(gen_errors(data, schema)))

        # flex flex_additionalProperties
        data['new_object'] = {'some': 'new object'}
        self.assertEqual(next(gen_errors(data, schema)).validator, 'additionalProperties')
        flex_schema_by_errs(schema, list(gen_errors(data, schema)))
        self.assertFalse(list(gen_errors(data, schema)))

        # flex complex types array
        data['object']['sub_object']['sub_array'] = [1, 2, '5', {'some': 'mixed value'}]
        self.assertEqual(next(gen_errors(data, schema)).validator, 'anyOf')
        flex_schema_by_errs(schema, list(gen_errors(data, schema)))
        self.assertFalse(list(gen_errors(data, schema)))


if __name__ == '__main__':
    unittest.main()



# encoding: utf-8
from __future__ import absolute_import, unicode_literals

from rest_framework import renderers


class BrowsableAPIRenderer(renderers.BrowsableAPIRenderer):

    def __init__(self, *args, **kwargs):
        super(BrowsableAPIRenderer, self).__init__(*args, **kwargs)
        self.template = 'apiview/api.html'


class JSONPRenderer(renderers.JSONRenderer):
    """
    Renderer which serializes to json,
    wrapping the json output in a callback function.
    """

    media_type = 'application/javascript'
    format = 'jsonp'
    callback_parameter = 'callback'
    default_callback = 'callback'
    charset = 'utf-8'

    @staticmethod
    def get_query_params(request):
        if hasattr(request, 'query_params'):
            params = request.query_params
        else:
            # DRF < 3.2
            params = request.QUERY_PARAMS

        return params

    def get_callback(self, renderer_context):
        """
        Determine the name of the callback to wrap around the json output.
        """
        request = renderer_context.get('request', None)
        params = request and self.get_query_params(request) or {}
        return params.get(self.callback_parameter, self.default_callback)

    def render(self, data, accepted_media_type=None, renderer_context=None):
        """
        Renders into jsonp, wrapping the json output in a callback function.

        Clients may set the callback function name using a query parameter
        on the URL, for example: ?callback=exampleCallbackName
        """
        renderer_context = renderer_context or {}
        callback = self.get_callback(renderer_context)
        json = super(JSONPRenderer, self).render(data, accepted_media_type,
                                                 renderer_context)
        return callback.encode(self.charset) + b'(' + json + b');'



#! usr/bin/env python
# encoding: utf-8

from __future__ import unicode_literals

from django.apps import AppConfig


class ApiViewConfig(AppConfig):
    name = 'apiview'

    def ready(self):
        # 后台filter自定义
        from django.contrib import admin
        from django.db import models
        from apiview.filters import (
            RelatedAutocompleteFilter, DateFieldListFilter,
        )

        def _autocomplete_filter_check(field):
            '''
            check field to use RelatedAutocompleteFilter
            '''
            if field.remote_field:
                return getattr(field.remote_field.model, 'AJAX_FILTER', True)
            elif isinstance(field, models.ForeignKey):
                return True
            return False

        admin.FieldListFilter.register(
            _autocomplete_filter_check,
            RelatedAutocompleteFilter,
            take_priority=True
        )

        admin.FieldListFilter.register(
            lambda field: isinstance(field, models.DateField),
            DateFieldListFilter,
            take_priority=True
        )

        # 防止关联查询
        class _RelatedFieldListFilter(admin.RelatedFieldListFilter):
            def __init__(self, field, request, params, model, model_admin, field_path):
                super(_RelatedFieldListFilter, self).__init__(field, request, params, model, model_admin, field_path)
                self.lookup_kwarg = '%s__exact' % field_path
                self.lookup_val = request.GET.get(self.lookup_kwarg)

        admin.FieldListFilter.register(
            lambda f: f.remote_field,
            _RelatedFieldListFilter,
            take_priority=True
        )



# encoding: utf-8
from __future__ import absolute_import, unicode_literals

from importlib import import_module

from django.conf import settings
from rest_framework.decorators import api_view, renderer_classes
from rest_framework.renderers import StaticHTMLRenderer
from rest_framework.response import Response

from .view import APIView
from .err_code import ErrCode


@api_view(["GET"])
@renderer_classes((StaticHTMLRenderer,))
def generate_api_js(request):
    # api.js?package_name=bbapi.views&ext_params=referral_code,version,device_id,channel
    tags = ErrCode.get_tags()
    content = ''
    content += "server:'%s', //服务地址\n" % request.build_absolute_uri("/")[:-1]
    for tag in tags:
        code_data = getattr(ErrCode, tag)
        content += '%s:%d, //%s\n' % (tag, code_data.code, code_data.message)
    ext_params_str = request.GET.get("ext_params", '')
    ext_params = set(ext_params_str.split(','))
    views = get_view_list()
    for view in views:
        func_name = view['url'].replace('/', '_').strip('_')
        str_args = ''
        str_data = ''
        for param in view['params'].keys():
            if param in ext_params:
                continue
            str_args += ', %s' % param
            str_data += "%s:%s," % (param, param)

        str_data = str_data[:-1]
        content += '''
// %s
%s: function(listener%s){
    var url = '%s';
    var data = {%s};
    this.conn(url, data, listener);
},''' % (view['name'], func_name, str_args, view['url'], str_data)
    return Response(content.encode("utf8"), content_type='text/plain;charset=utf-8')


def get_url(head, urlpattern):
    url = getattr(urlpattern, 'pattern', urlpattern).regex.pattern
    ret = head + url.rstrip("$?").lstrip('^')
    return ret.replace('//', '/')


# def get_package_list(urlpattern, ext_params, head):
#     ret = ''
#     if hasattr(urlpattern, 'callback') \
#         and urlpattern.callback \
#         and hasattr(urlpattern.callback, 'view_class') \
#         and issubclass(urlpattern.callback.view_class, APIView):
#         viewclass = urlpattern.callback.view_class
#         params = viewclass._meta.param_fields
#         name = viewclass.__name__
#         # url = viewclass._meta.path[:-1]
#         url = head.replace('//', '/').rstrip('/')
#         func_name = url.replace('/', '_').strip('_')
#         str_args = ''
#         str_data = ''
#         for param in params.keys():
#             if param in ext_params:
#                 continue
#             str_args += ', %s' % param
#             str_data += "%s:%s," % (param, param)
#         ret += '''
#             // %s
#             %s: function(listener%s){
#                 var url = api.server + '%s';
#                 var data = {%s};
#                 api.conn(url, data, listener);
#             },''' % (name, func_name, str_args, url, str_data)

#     if hasattr(urlpattern, 'url_patterns'):
#         for pattern in urlpattern.url_patterns:
#             ret += get_package_list(pattern, ext_params, get_url(head, pattern))

#     return ret

def get_view_list(urlpattern=None, head='/'):
    ret = []
    if urlpattern is None:
        rooturl = import_module(settings.ROOT_URLCONF)
        for urlpattern in rooturl.urlpatterns:
            ret += get_view_list(urlpattern, get_url(head, urlpattern))
        return ret
    if hasattr(urlpattern, 'callback') \
            and urlpattern.callback \
            and hasattr(urlpattern.callback, 'view_class') \
            and issubclass(urlpattern.callback.view_class, APIView):
        retdict = dict()
        viewclass = urlpattern.callback.view_class
        retdict['viewclass'] = viewclass
        retdict['params'] = viewclass._meta.param_fields
        retdict['name'] = getattr(viewclass, 'name', viewclass.__name__)
        retdict['url'] = head.replace('//', '/').rstrip('/')
        ret.append(retdict)
        # func_name = url.replace('/', '_').strip('_')
        # str_args = ''
        # str_data = ''
        # for param in params.keys():
        #     if param in ext_params:
        #         continue
        #     str_args += ', %s' % param
        #     str_data += "%s:%s," % (param, param)
        # ret += '''
        #     // %s
        #     %s: function(listener%s){
        #         var url = api.server + '%s';
        #         var data = {%s};
        #         api.conn(url, data, listener);
        #     },''' % (name, func_name, str_args, url, str_data)

    if hasattr(urlpattern, 'url_patterns'):
        for pattern in urlpattern.url_patterns:
            ret += get_view_list(pattern, get_url(head, pattern))

    return ret



# encoding: utf-8
from __future__ import absolute_import, unicode_literals

import six
import base64
import decimal
import datetime
from functools import partial
import hashlib
import imghdr
import json
import logging
import math
import os
import random
import re
import string
import urllib
import uuid
import posixpath

from django.conf import settings
from django.core.files.uploadedfile import SimpleUploadedFile
from django.core.files.storage import default_storage
from django.core.mail.message import EmailMultiAlternatives
from django.utils.encoding import force_str, force_text, force_bytes

from . import validators


DATE_FORMAT = getattr(settings, 'DATE_FORMAT', None)
TIME_FORMAT = getattr(settings, 'TIME_FORMAT', '%H:%M:%S')
DATETIME_FORMAT = getattr(settings, 'DATETIME_FORMAT', None)
ASCII_CHARS = 'ABCDEFGHJKLMNPQRSTUVWXYZ'
DIGIT_CHARS = '23456789'
CHARS = DIGIT_CHARS + ASCII_CHARS


def id_generator(size=6, chars=CHARS):
    return ''.join(random.sample(chars, int(size)))


def ids_generator(size=6, amount=10, chars=CHARS):
    return [id_generator() for _ in range(amount)]


def unique_ids_generator(ids, size=6, amount=10, chars=CHARS):
    local_ids = []
    count = 0
    while count < amount:
        local_id = id_generator()
        if local_id not in ids and local_id not in local_ids:
            local_ids.append(local_id)
            count += 1
    return local_ids


# 用于ImageFiled的upload_to参数，生成文件名，useuuid＝True使用uuid
# secs为字段列表组成文件名
def uuid_filename_generator(instance, filename, path_format):
    filetype = os.path.splitext(filename)[1]
    if len(filetype) == 0:
        filetype = '.'

    basename = ''.join((force_text(uuid.uuid4()), filetype))

    dirname = force_text(datetime.datetime.now().strftime(force_str(path_format)))
    filename = posixpath.join(dirname, basename)
    return default_storage.generate_filename(filename)
    # expandpath = datetime.datetime.strftime(datetime.datetime.now(), path)
    # return os.path.join(expandpath, filename)


def get_uuid_filename(path_format):
    return partial(uuid_filename_generator, path_format=path_format)


# 判断是否手机号
def IsMobileNumber(mobile):
    return validators.mobile.is_valid(mobile)


g_weekdaystr = ('周一', '周二', '周三', '周四', '周五', '周六', '周日')


def get_weekday_str(date):
    return g_weekdaystr[date.weekday()]


def safeMobile(mobile):
    if len(mobile) != 11:
        return ''
    return mobile[:3]+'****'+mobile[-4:]


def priceFormat(price, decimal_places=2):
    return (('%%.%df' % decimal_places) % price).strip('0').strip('.')

# def getResponseErr(ecode, loginfo, **kwargs):
#     from errDef import errResponse
#     from rest_framework.response import Response
#     err = errResponse(ecode, kwargs)
#     logging.debug(loginfo, err)
#     return(Response(err))


# 定义枚举
class Enum(object):
    def __init__(self, **kwargs):
        for attr, val in kwargs.items():
            setattr(self, attr, val)


def enum(**enums):
    return Enum(**enums)


def safe_loadjson(jstr):
    retobj = None
    try:
        retobj = json.loads(jstr)
    except Exception:
        reportExceptionByMail("json load error")

    return retobj

# def prase_url(request, value):
#     if not value or value.strip() == '':
#         return ''
#     if '://' in value:
#         return value
#     if request:
#         return request.build_absolute_uri(value)
#     else:
#         return config.SL_HTTP_HOST + value


def appendMediaURL(value):
    if hasattr(value, 'url'):
        return value.url

    if not value or value.strip() == '':
        return ''
    else:
        return os.path.join(settings.MEDIA_URL, value)

# def real_url(request, pic_field):
#     if not pic_field:
#         return ''
#     if hasattr(pic_field, 'name'):
#         pic_field = pic_field.name
#     return prase_url(request, appendMediaURL(pic_field))


def hash_sha1(str):
    if not str or str.strip() == '':
        return None

    mdTemp = hashlib.sha1()
    mdTemp.update(str)

    return mdTemp.hexdigest()


def getServerIP():
    import socket
    ip = '0.0.0.0'
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.connect(('114.114.114.114', 53))
        ip = s.getsockname()[0]
    except Exception:
        reportExceptionByMail('get server ip failed')

    return ip


LST_COLOR_CHARS = ('白', '黑', '红', '绿', '蓝', '黄', '银', '灰', '粉', '紫', '橙', '棕', '香槟', '咖啡', '金')


def colorValidate(color):
    if not color or len(color) == 0:
        return '请输入颜色'

    if len(color) > 10:
        return '颜色输入过长'

    if 0 not in [string.find(color, c) == -1 for c in LST_COLOR_CHARS]:
        return '请用中文正确输入颜色'

    return ''


def sendEmail(subject, add_to, html_content):
    from django.conf import settings

    if isinstance(add_to, six.string_types):
        addr1 = add_to
        add_to = list()
        add_to.append(addr1)
    subject = force_text(subject)
    if subject.find(settings.EMAIL_SUBJECT_PREFIX) < 0:
        subject = settings.EMAIL_SUBJECT_PREFIX + subject

    # send_mail(subject, html_content, settings.EMAIL_HOST_USER,
    # add_to, fail_silently=False)

    # text_content = 'This is an important message.'
    # html_content = '<p>This is an <strong>important</strong> message.</p>'

    msg = EmailMultiAlternatives(subject, html_content, settings.EMAIL_HOST_USER, add_to)
    msg.attach_alternative(html_content, "text/html")
    msg.send()


def earth_distance(lat1, lon1, lat2, lon2):
    radlat1 = (math.pi/180)*lat1
    radlat2 = (math.pi/180)*lat2

    radlon1 = (math.pi/180)*lon1
    radlon2 = (math.pi/180)*lon2

    a = radlat1-radlat2
    b = radlon1-radlon2
    s = 2 * math.asin(math.sqrt(math.pow(math.sin(a / 2), 2) +
                                math.cos(radlat1) *
                                math.cos(radlat2) *
                                math.pow(math.sin(b / 2), 2)))
    earth_radius = 6378.137
    s = s * earth_radius
    if s < 0:
        return -s
    else:
        return s


# 返回一天的最开始时间和返回时间，组成元组
def dayRange(day):
    day_min = datetime.datetime.combine(day, datetime.time.min)
    day_max = datetime.datetime.combine(day, datetime.time.max)
    return day_min, day_max


# 返回一月的最开始时间和返回时间，组成元组
def monthRange(day):
    import calendar

    lastday = calendar.monthrange(day.year, day.month)

    fromday = datetime.datetime(day.year, day.month, 1)
    today = datetime.datetime(day.year, day.month, lastday[1])

    return fromday, today


def rsa_decrypt(srcStr, prikey):
    import rsa
    res_data = base64.b64decode(srcStr)
    lendata = len(res_data)
    patch = lendata % 128
    if patch > 0:
        patch = 128 - patch
        res_data += '\0' * patch
    secs = len(res_data) / 128
    result = ''
    for sec in range(0, secs):
        result += rsa.decrypt(res_data[sec*128:sec*128+128], prikey)

    return result


def querystring2dict(querystring):
    from six.moves.urllib_parse import urlparse
    resultDict = dict((k, v if len(v) > 1 else v[0]) for k, v in urlparse.parse_qs(querystring).items())
    return resultDict


# 获取严重级别颜色
def getSeverityColor(val, step, shreshold=0):
    level = (val - shreshold) / step
    if level > 255:
        level = 255
    if level < 0:
        level = 0

    return '#%02x%02x00' % (level, 255-level)


# 时间差对象转字符串
def timedelta2Str(dt, ms=False):
    secs = dt.total_seconds()
    sign = "-" if secs < 0 else ""

    if secs < 0:
        dt = datetime.timedelta(seconds=-secs)
    sdt = str(dt)
    if not ms:
        sdt = sign + sdt.split('.')[0]
    return sdt


_logger_exception = logging.getLogger('exception')


def reportExceptionByMail(msg, *args, **kwargs):

    _logger_exception.exception(msg, *args, **kwargs)


def yields(listobj, n=1000):
    i = 0
    len_ = len(listobj)
    while n*i < len_:
        yield listobj[n*i:(i+1)*n]
        i += 1


# sql执行结果cursor转换为字典
def getCursorDict(c):
    fnames = [fd[0] for fd in c.description]

    result = list()
    for row in c:
        result.append(dict(zip(fnames, row)))

    return result


def filter_emoji(desstr, restr=''):
    '''
    过滤表情
    '''
    try:
        co = re.compile('[\U00010000-\U0010ffff]')
    except re.error:
        co = re.compile('[\uD800-\uDBFF][\uDC00-\uDFFF]')
    return co.sub(restr, desstr)


def str_2_emoji(emoji_str):
    if six.PY3:
        from html.parser import HTMLParser
    else:
        from HTMLParser import HTMLParser
    '''
    把字符串转换为表情
    '''
    if not emoji_str:
        return emoji_str
    h = HTMLParser.HTMLParser()
    emoji_str = h.unescape(h.unescape(emoji_str))
    # 匹配u"\U0001f61c"和u"\u274c"这种表情的字符串
    co = re.compile(r"u[\'\"]\\[Uu]([\w\"]{9}|[\w\"]{5})")
    pos_list = []
    result = emoji_str
    # 先找位置
    for m in co.finditer(emoji_str):
        pos_list.append((m.start(), m.end()))
    # 根据位置拼接替换
    for pos in range(len(pos_list)):
        if pos == 0:
            result = emoji_str[0:pos_list[0][0]]
        else:
            result = result + emoji_str[pos_list[pos-1][1]:pos_list[pos][0]]
        result = result + eval(emoji_str[pos_list[pos][0]:pos_list[pos][1]])
        if pos == len(pos_list) - 1:
            result = result + emoji_str[pos_list[pos][1]:len(emoji_str)]
    return result


def datetime2timestamp(dtime):
    import time
    if not dtime:
        return 0
    return int(time.mktime(dtime.timetuple()))


def timestamp2datetime(timestamp):
    return datetime.datetime.fromtimestamp(timestamp)


def datetime2str(dtime, format):
    return dtime.strfomrt(format)


def format_res_data(data):

    if hasattr(data, 'items'):
        for k, v in data.items():
            data[k] = format_res_data(v)
        return data
    elif isinstance(data, (tuple, list)):
        tmp = []
        for item in data:
            tmp.append(format_res_data(item))
        return tmp
    elif isinstance(data, decimal.Decimal):
        return six.text_type(data)
    else:
        timeformat = None
        canstamp = False
        if isinstance(data, datetime.datetime):
            canstamp = True
            timeformat = DATETIME_FORMAT
        elif isinstance(data, datetime.date):
            canstamp = True
            timeformat = DATE_FORMAT
        elif isinstance(data, datetime.time):
            canstamp = False
            timeformat = TIME_FORMAT
        if timeformat is not None:
            return data.strftime(timeformat)
        elif canstamp:
            return datetime2timestamp(data)
        else:
            return data


# 把字典内容赋值给对象属性
def copy_dict2obj(dict, obj):
    if not dict or not obj:
        return
    for key in dict.keys():
        setattr(obj, key, dict[key])


def percentage(i1, i2):
    if i2 == 0:
        return None
    return round((i1 * 1.0) / (i2 * 100.0), 2)


def time_to_second(t):
    if not t or not isinstance(t, datetime.time):
        return 0
    return t.hour * 3600 + t.minute * 60 + t.second


# des加解密
def des_encode(inputdes, key):
    from . import des
    try:
        des_enc = des.des(key[:8], padmode=des.PAD_PKCS5)
        inputdes = force_bytes(inputdes)
        return base64.b64encode(des_enc.encrypt(inputdes))
    except Exception:
        import inspect
        msg = "des_encode error"
        msg += ","
        inspect.stack()
        msg += ' => '.join('%s-%s:%d' % (item[1], item[3], item[2]) for item in inspect.stack())
        reportExceptionByMail(msg)
        return None


def des_decode(inputdes, key):
    from . import des

    debase64 = None
    try:
        debase64 = base64.b64decode(inputdes)
    except Exception:
        reportExceptionByMail("'" + inputdes + "'")
        return None

    try:
        des_dec = des.des(key[:8], padmode=des.PAD_PKCS5)
        return des_dec.decrypt(debase64)
    except Exception:
        import inspect
        msg = "des_decode error"
        msg += ","
        inspect.stack()
        msg += ' => '.join('%s-%s:%d' % (item[1], item[3], item[2]) for item in inspect.stack())
        reportExceptionByMail(msg)
        return None


def pad(s, pad_size):
    return s + (pad_size - len(s) % pad_size) * chr(pad_size - len(s) % pad_size)


def unpad(s):
    return s[0:-ord(s[-1])]


def aes_encode(inputdes, key):
    from Crypto.Cipher import AES
    iv = str(bytearray(16))
    cipher = AES.new(key[:16], AES.MODE_ECB, iv)
    inputdes = force_bytes(inputdes)
    return base64.b64encode(cipher.encrypt(pad(inputdes, 16)))


def aes_decode(inputdes, key):
    from Crypto.Cipher import AES
    iv = str(bytearray(16))
    cipher = AES.new(key[:16], AES.MODE_ECB, iv)
    return unpad(cipher.decrypt(base64.b64decode(inputdes)))


def get_host_from_url(url):
    protocol, s1 = urllib.splittype(url)
    host, s2 = urllib.splithost(s1)
    host, port = urllib.splitport(host)
    if port is None:
        port = 80
    return host, port


def date_format(date_str):
    try:
        _date = datetime.datetime.strptime(date_str, '%Y-%m-%d').date()
        return _date
    except Exception:
        return None


def month_from_date(day):
    import calendar
    if isinstance(day, six.string_types):
        day = datetime.datetime.strptime(day, '%Y-%m-%d').date()
    month_code = day.strftime("%Y-%m")
    wday, monthRange = calendar.monthrange(day.year, day.month)
    month_begin = datetime.date(day.year, day.month, 1)
    month_end = datetime.date(day.year, day.month, monthRange)
    return month_code, month_begin, month_end


def week_from_date(day):
    if isinstance(day, six.string_types):
        day = datetime.datetime.strptime(day, '%Y-%m-%d').date()
    from dateutil.relativedelta import relativedelta, MO, SU
    weekbegin = day + relativedelta(weekday=MO(-1))
    weekend = day + relativedelta(weekday=SU(1))
    week_code = '%d_%d' % (day.isocalendar()[0], day.isocalendar()[1])
    return week_code, weekbegin, weekend


def fields_verify(model_admin, field_names, only_fields=False):
    model = model_admin.model
    real_field_names = []
    for field_name in field_names:
        try:
            if model._meta.get_field(field_name):
                real_field_names.append(field_name)
        except Exception:
            if not only_fields:
                if hasattr(model_admin, field_name):
                    real_field_names.append(field_name)
    return real_field_names


def get_qrimgurl_by_content(request, content, **kwargs):
    import qrcode
    import os
    from hashlib import md5
    expandpath = datetime.datetime.now().strftime("qr/%Y/%Y%m%d/")
    filename = "%s%s.png" % (expandpath, md5(content).hexdigest())
    file = os.path.join(settings.MEDIA_ROOT, filename)
    path, name = os.path.split(file)
    if not os.path.exists(path):
        os.makedirs(path)
    with open(file, 'wb') as fp:
        qrcode.make(content, **kwargs).save(fp)
    qrurl = request.build_absolute_uri('/')[:-1] + settings.MEDIA_URL + filename
    return qrurl


def check_time(time_to_check, on_time, off_time):
    if on_time > off_time:
        if time_to_check > on_time or time_to_check < off_time:
            return True
    elif on_time < off_time:
        if time_to_check > on_time and time_to_check < off_time:
            return True
    elif time_to_check == on_time:
        return True
    return False


def request_ip(request):
    if 'HTTP_X_FORWARDED_FOR' in request.META:
        ip = request.META['HTTP_X_FORWARDED_FOR']
    else:
        ip = request.META['REMOTE_ADDR']
    return ip


# 邮件签名, 包括IP地址,时间, 是否生产环境等
# def mail_remark():

#     remark = '<br>IP:%s <br>时间:(%s)' % (settings.SERVER_IP, datetime.datetime.now())
#     remark += '<br>调试模式：{b}'.format(b='是' if settings.DEBUG else '否')
#     remark += '<br>生产环境：{b}'.format(b='是' if config.IS_PRODUCTION else '否')

#     return remark

def decode_base64_image(base_str):
    image = None
    try:
        missing_padding = 4 - len(base_str) % 4
        if missing_padding:
            base_str += b'=' * missing_padding
        base_str = base64.b64decode(base_str)
        if len(base_str) > 0:
            image = SimpleUploadedFile(name='pic.jpg', content=base_str, content_type='image/jpeg')
            if not imghdr.what(image):
                image = None
    except Exception as e:
        logging.error(e)
        image = None
    return image



# encoding: utf-8
from __future__ import absolute_import, unicode_literals

from django.contrib import admin
from django.templatetags.static import static

from apiview import admintools
from apiview import model


class BaseAdmin(admintools.ProxyModelAdmin):

    class Media:
        js = (static('apiview/js/filter.js'),)
        css = {
            'all': (static('apiview/css/filter.css'),),
        }


def site_register(model_or_iterable, admin_class=None, site=None, **options):
    if site is None:
        site = admin.site
    if admin_class is None:
        admin_class = BaseAdmin
    if not isinstance(model_or_iterable, (list, set, tuple)):
        model_or_iterable = [model_or_iterable]
    for m in model_or_iterable:
        if issubclass(m, model.BaseModel):
            m2m_fields = []
            search_fields = list(m.search_fields())
            generic_foreign_keys = []
            generic_relations = []
            list_display = []

            for key, field in m._meta._forward_fields_map.items():
                if key != field.name:
                    continue
                if field.is_relation:
                    if field.one_to_many:
                        generic_relations.append(field.name)
                    elif field.many_to_many:
                        m2m_fields.append(field.name)
                    elif field.many_to_one and hasattr(field.remote_field, 'model') and field.remote_field.model:
                        generic_foreign_keys.append(field.name)
                    elif field.one_to_one and hasattr(field.remote_field, 'model') and field.remote_field.model:
                        generic_foreign_keys.append(field.name)

            filter_horizontal = m2m_fields
            raw_id_fields = generic_foreign_keys
            if admin_class is not None and admin_class != BaseAdmin:
                _list_display = admin_class.list_display
                if _list_display is None:
                    list_display = ()
                elif len(_list_display) > 0:
                    list_display = _list_display
                _search_fields = admin_class.search_fields
                if _search_fields is None:
                    search_fields = ()
                elif len(_search_fields) > 0:
                    search_fields = _search_fields
                _raw_id_fields = admin_class.raw_id_fields
                if _raw_id_fields is None:
                    raw_id_fields = ()
                elif len(_raw_id_fields) > 0:
                    raw_id_fields = _raw_id_fields
                _filter_horizontal = admin_class.filter_horizontal
                if _filter_horizontal is None:
                    filter_horizontal = ()
                elif len(_filter_horizontal) > 0:
                    filter_horizontal = _filter_horizontal

            if len(search_fields) > 0:
                if 'search_fields' not in options:
                    options['search_fields'] = search_fields
            if len(raw_id_fields) > 0:
                if 'raw_id_fields' not in options:
                    options['raw_id_fields'] = raw_id_fields
            if len(filter_horizontal) > 0:
                if 'filter_horizontal' not in options:
                    options['filter_horizontal'] = filter_horizontal

            if 'list_display' in options:
                list_display = list(list_display) + list(options['list_display'])

            if len(list_display) > 0:
                for attr in list_display:
                    if '__' in attr and not attr.startswith('_'):
                        _boolean, _short_description = m.get_child_field(attr)
                        key = '{}_boolean'.format(attr)
                        if not hasattr(admin_class, key) and key not in options:
                            options[key] = _boolean
                        key = '{}_short_description'.format(attr)
                        if not hasattr(admin_class, key) and key not in options:
                            options[key] = _short_description

            autocomplete_lookup_fields = options.get('autocomplete_lookup_fields',
                                                     getattr(admin_class, 'autocomplete_lookup_fields', {}))
            if 'raw_id_fields' in options and 'fk' not in autocomplete_lookup_fields:
                autocomplete_lookup_fields['fk'] = options['raw_id_fields']
            # if len(m2m_fields) > 0 and 'm2m' not in autocomplete_lookup_fields:
            #     autocomplete_lookup_fields['m2m'] = m2m_fields
            options['autocomplete_lookup_fields'] = autocomplete_lookup_fields

        site.register(m, admin_class, **options)



# encoding: utf-8

"""
Wrap django form fields to use `omit`, `default` instead of `required`

重载Django fields以获取对omit、default等特性的支持

注意：
    继承时，如果对__init__, to_python, clean 方法进行重写时，如果不调用父级
    方法，则必须使用wrap_field(method=(<上述重写的方法名>...))对子类进行处理，
    参看BooleanField的处理
"""

from __future__ import unicode_literals, absolute_import

import six

from functools import wraps, partial
from django.db import models
from django.forms import fields
from django.forms.models import fields_for_model
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _
from django.utils.encoding import force_text
from apiview import utility
# to prevent Importation-Examination
from django.forms.fields import *  # NOQA

from apiview import validators
from .widgets import BooleanInput, NullBooleanSelect


class _Empty(object):
    def __nonzero__(self):
        return False

    def __str__(self):
        return '<empty>'


empty = _Empty()


def formfield(field, **kwargs):
    kwargs.setdefault('validators', field.validators)
    if isinstance(field, (
            models.AutoField,
            models.PositiveIntegerField,
            models.PositiveSmallIntegerField)):
        return fields.IntegerField(min_value=0, **kwargs)
    else:
        return globals().get(field.get_internal_type(), fields.CharField)(**kwargs)


def field_for_model(model, field, widget=None, localize=None, help_text=None, error_messages=None):
    return fields_for_model(model, [field], None, widget and {field: widget}, formfield,
                            localize is not None and {field: localize} or localize, None,
                            help_text and {field: help_text},
                            error_messages and {field: error_messages})[field]


def field_info(field):

    if isinstance(field, (fields.BooleanField, fields.NullBooleanField)):
        return '0, false; 1, true'

    if hasattr(field, 'choices'):
        return '; '.join([': '.join(map(force_text, choice)) for choice in field.choices])
    help_text_li = []
    for att in ['max_value', 'min_value', 'max_length', 'min_length', 'max_digits', 'sep', 'seps', 'regex']:
        val = getattr(field, att, None)
        if val is not None:
            help_text_li.append('%s: %s' % (att, val))
    if hasattr(field, 'fields'):
        h_text = []
        for f in field.fields:
            if isinstance(f, fields.Field):
                h_text.append("%s" % f.__class__.__name__)
            elif type(f) == type:
                h_text.append("%s" % f.__name__)
            else:
                h_text.append('%s' % f)
        help_text_li.append("fields : (%s)" % ",".join(h_text))
    if hasattr(field, 'field'):
        if isinstance(field.field, fields.Field):
            help_text_li.append("field : %s" % field.field.__class__.__name__)
        elif type(field.field) == type:
            help_text_li.append('field : %s' % field.field.__name__)
        else:
            help_text_li.append('field : %s' % field.field)
    return '; '.join(help_text_li)


def _wrap_field(fieldclass, methods=()):
    '''rewrite the __init__, to_python and clean methods to add more features'''
    init_method = fieldclass.__init__

    @wraps(init_method)
    def __init__(self, *args, **kwargs):
        self.type_name = kwargs.pop('type_name', type(self).__name__)
        if self.type_name.endswith("Field"):
            self.type_name = self.type_name[:-5]
        self.omit = kwargs.pop('omit', empty)
        self.default = kwargs.pop('default', empty)
        if self.omit is not empty:
            kwargs['required'] = False
        elif self.default is not empty:
            kwargs.setdefault('required', True)
        init_method(self, *args, **kwargs)
        self.field_info = field_info(self)

    to_python_method = fieldclass.to_python

    @wraps(to_python_method)
    def to_python(self, value):
        if (value in self.empty_values
                and self.omit is not empty):
            if callable(self.omit):
                return self.omit()
            return self.omit

        value = to_python_method(self, value)

        return value

    clean_method = fieldclass.clean

    @wraps(clean_method)
    def clean(self, *args, **kwargs):
        try:
            return clean_method(self, *args, **kwargs)
        except ValidationError:
            if self.default is not empty:
                if callable(self.default):
                    return self.default()
                return self.default
            raise

    attrs = {
        '__init__': __init__,
        'to_python': to_python,
        'clean': clean,
    }
    if methods:
        all_methods = set(attrs.keys())
        methods = set(methods)
        if methods - all_methods:
            raise TypeError('Unexpected method to be reload: %s' % list(methods - all_methods))
        for item in all_methods - methods:
            attrs.pop(item)

    return type(fieldclass)(fieldclass.__name__, (fieldclass, ), attrs)


def wrap_field(methods=()):
    '''make _wrap_field as a decorator'''
    return partial(_wrap_field, methods=methods)


for _fieldclass_name in fields.__all__[1:]:
    six.exec_('{0} = _wrap_field(fields.{0})'.format(_fieldclass_name))


# overwrite BooleanField and NullBooleanField to accept 0,1
@wrap_field(methods=('to_python', ))
class BooleanField(BooleanField):  # NOQA
    widget = BooleanInput

    default_error_messages = {
        'invalid': _('Enter a valid value.'),
    }

    def to_python(self, value):
        if isinstance(value, six.string_types):
            value = value.lower()
            if value in ('false', '0'):
                value = False
            elif value in ('true', '1'):
                value = True
        return value

    def validate(self, value):
        if value in self.empty_values:
            raise ValidationError(self.error_messages['required'], code='required')
        if not isinstance(value, bool):
            raise ValidationError(self.error_messages['invalid'], code='invalid')


class NullBooleanField(NullBooleanField):  # NOQA
    widget = NullBooleanSelect


class MobileField(CharField):  # NOQA
    default_validators = [validators.mobile]


class LongitudeField(FloatField):  # NOQA
    def __init__(self, max_value=180.0, min_value=-180.0, *args, **kwargs):
        super(LongitudeField, self).__init__(
            max_value=max_value, min_value=min_value, *args, **kwargs
        )


class LatitudeField(FloatField):  # NOQA
    def __init__(self, max_value=90.0, min_value=-90.0, *args, **kwargs):
        super(LatitudeField, self).__init__(
            max_value=max_value, min_value=min_value, *args, **kwargs
        )


class SplitCharField(CharField):  # NOQA
    """Split string value with given sep or seps
    """

    default_field = fields.CharField()

    def __init__(self, *args, **kwargs):
        self.sep = kwargs.pop('sep', ',')
        self.field = kwargs.pop('field', self.default_field)
        super(SplitCharField, self).__init__(*args, **kwargs)

    def clean(self, value):
        value = super(SplitCharField, self).clean(value)
        if value:
            return list(map(self.field.clean, value.split(self.sep)))
        else:
            return []


class TimestampField(IntegerField):  # NOQA
    def to_python(self, value):
        v = super(TimestampField, self).to_python(value)
        if v is None:
            return None
        return utility.timestamp2datetime(v)


class PairCharField(CharField):  # NOQA
    """Split string value with given seps"""
    default_error_messages = {
        'invalid': _('Enter a valid value.'),
        'unpaired': _('Enter a valid value.'),
    }

    default_field = (fields.CharField(), fields.CharField())

    def __init__(self, *args, **kwargs):
        self.seps = kwargs.pop('seps', ('|', '.'))
        self.fields = kwargs.pop('fields', self.default_field)
        super(PairCharField, self).__init__(*args, **kwargs)

    def clean(self, value):
        value = super(PairCharField, self).clean(value)
        if value:
            value_list = []
            for val0, val1 in self.gen_split(value):
                value_list.append(
                    (self.fields[0].clean(val0), self.fields[1].clean(val1))
                )
            return value_list
        else:
            return []

    def gen_split(self, value, idx=0):
        if not value:
            return
        if idx == len(self.seps) - 1:
            pair = value.split(self.seps[idx])
            if len(pair) != 2:
                raise ValidationError(
                        self.error_messages['unpaired'],
                        code='unpaired',
                        params={'seps': self.seps},
                    )
            yield value.split(self.seps[idx])
        else:
            for item in value.split(self.seps[idx]):
                for ret in self.gen_split(item, idx + 1):
                    yield ret



# encoding: utf-8
from __future__ import absolute_import, unicode_literals


class CodeData(object):

    def __init__(self, code, tag, message):
        self.code = code
        self.message = message
        self.tag = tag

    def __str__(self):
        return str(self.code)

    def __eq__(self, other):
        if isinstance(other, CodeData):
            return other.code == self.code
        elif isinstance(other, type(self.code)):
            return other == self.code
        else:
            return super(CodeData, self).__eq__(other)

    def get_res_dict(self, **kwargs):
        ret = dict(kwargs)
        ret['code'] = self.code
        if 'message' not in ret:
            ret['message'] = self.message
        return ret


class Code(object):

    def __init__(self, code_define):
        codes = set()
        self._list = list()
        self._dict = dict()
        self._tags = list()
        for tag, code, message in code_define:
            assert code not in codes and not hasattr(self, tag)
            setattr(self, tag, CodeData(code, tag, message))
            codes.add(code)
            self._tags.append(tag)
            self._list.append((code, message))
            self._dict[code] = message

    def get_list(self):
        return self._list

    def get_dict(self):
        return self._dict

    def get_tags(self):
        return self._tags



# encoding: utf-8
from __future__ import absolute_import, unicode_literals

from django.template import Library


register = Library()


@register.inclusion_tag("apiview/scripts.html", takes_context=True)
def apiview(context):
    return context



# encoding: utf-8
from __future__ import absolute_import, unicode_literals

import pickle
import base64
import datetime

try:
    from channels.generic.websocket import JsonWebsocketConsumer
except Exception:
    from channels.generic.websockets import JsonWebsocketConsumer

from channels.handler import AsgiRequest
from django.urls import Resolver404, get_resolver


class ApiViewConsumer(JsonWebsocketConsumer):
    http_user = True
    # Set to True if you want it, else leave it out
    strict_ordering = False
    CONNECT_CONTENT_KEY = 'CONNECT_CONTENT'

    def __init__(self, message, **kwargs):
        super(ApiViewConsumer, self).__init__(message, **kwargs)
        self.message.reply_channel.send({"accept": True})

    @property
    def request(self):
        if not hasattr(self, "_request"):
            msg = self.message
            connect_content = msg.channel_session.get(self.CONNECT_CONTENT_KEY, None)
            if connect_content is not None:
                connect_content = pickle.loads(base64.b64decode(connect_content.encode()))
                msg = msg.copy()
                for k in connect_content.keys():
                    if k not in msg:
                        msg[k] = connect_content[k]
            request = AsgiRequest(msg)
            setattr(self, "_request", request)
        return getattr(self, "_request")

    @staticmethod
    def get_response(request, path, data, user, reply_channel):
        from .view import APIView
        resolver = get_resolver()
        resolver_match = resolver.resolve(path)
        callback, callback_args, callback_kwargs = resolver_match
        if not hasattr(callback, 'view_class') or not issubclass(callback.view_class, APIView):
            raise Resolver404
        request.resolver_match = resolver_match
        response = callback(request, is_ws=True, ws_data=data, ws_user=user,
                            ws_reply_channel=reply_channel, *callback_args, **callback_kwargs)
        return response

    def connection_groups(self, **kwargs):
        return ["apiview"]

    def connect(self, message, **kwargs):
        super(ApiViewConsumer, self).connect(message, **kwargs)
        content = base64.b64encode(pickle.dumps(message.content)).decode()
        message.channel_session[self.CONNECT_CONTENT_KEY] = content

    def receive(self, content, **kwargs):
        path = content.get("path", None)
        reqid = content.get("reqid", None)
        data = content.get("data", None)
        res = dict()
        try:
            response = self.get_response(self.request, path, data,
                                         getattr(self.message, "user", None), self.message.reply_channel)
            res['data'] = response.data
            res['status_code'] = response.status_code
        except Resolver404:
            res['status_code'] = 404
        except Exception:
            res['status_code'] = 500
        res['server_time'] = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        res['reqid'] = reqid
        self.send(res)

    def disconnect(self, message, **kwargs):
        pass



# encoding: utf-8
"""
Definition of the rest framework compatible view class
"""
from __future__ import absolute_import

import six

from django import forms
from rest_framework.views import APIView
from rest_framework.response import Response

from .base import ViewMetaclass, ViewOptions
from .param import Param


class APIViewOptions(ViewOptions):
    """APIView options class

    Extend options:
    * wrappers          view's wrappers generate with the nearest-first logic,
                        this attribute will generate all parents' wrappers
    """

    def __init__(self, options=None, parent=None):
        super(APIViewOptions, self).__init__(options, parent)
        self.wrappers = list(reversed(getattr(options, 'wrappers', ())))

        if self.parent:
            self.wrappers.extend(parent._meta.wrappers)

    def wrap_view(self, view):
        for wrapper in self.wrappers:
            view = wrapper(view)
        return view


class View(six.with_metaclass(ViewMetaclass, APIView)):
    """
    Rest-Framework compatible view
    """

    option_class = APIViewOptions

    def __init__(self, *args, **kwargs):
        super(View, self).__init__(*args, **kwargs)
        self.http_method_names = self.http_method_names + ["ws", ]

    def get_renderer_context(self):
        context = super(View, self).get_renderer_context()
        context.update({
            'view_options': self._meta,
        })
        return context

    def dispatch(self, request, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs
        ___ = request.body  # NOQA
        request = self.initialize_request(request, *args, **kwargs)
        self.request = request
        self.headers = self.default_response_headers
        self.is_ws = kwargs.get("is_ws", False)
        self.ws_data = kwargs.get("ws_data", {})
        self.ws_reply_channel = kwargs.get("ws_reply_channel", None)
        ws_user = kwargs.get("ws_user", None)
        if ws_user is not None:
            request.user = ws_user
        try:
            self.initial(request, *args, **kwargs)
            handler_name = request.method.lower()
            if self.is_ws:
                handler_name = "ws"
            if handler_name in self.http_method_names:
                # opts = self._meta
                if hasattr(self, handler_name):
                    handler = getattr(self, handler_name)
                    if not getattr(handler, '_decorated', False):
                        decorated_handler = self._meta.decorate_handler(handler)
                        if decorated_handler is not handler:
                            decorated_handler.__dict__['_decorated'] = True
                            setattr(self, handler_name, decorated_handler)
                            handler = decorated_handler
                else:
                    handler = self.http_method_not_allowed

                request.params = Param(self, request, kwargs, self.is_ws, self.ws_data)
                request._request.params = request.params
                response = handler(request, *args, **kwargs)
            else:
                handler = self.http_method_not_allowed
                response = handler(request, *args, **kwargs)
        except forms.ValidationError as exc:
            response = self.handle_param_errors(exc)
        except Exception as exc:
            response = self.handle_exception(exc)

        self.response = self.finalize_response(request, response, *args, **kwargs)
        return self.response

    def handle_param_errors(self, exc):
        if hasattr(exc, 'error_dict'):
            return Response(exc.error_dict, status=400)
        else:
            return Response({'*': exc}, status=400)

    @classmethod
    def as_view(cls, *args, **kwargs):
        view = super(View, cls).as_view(*args, **kwargs)
        return cls._meta.wrap_view(view)



# encoding: utf-8
from __future__ import absolute_import, unicode_literals

# JSONSCHEMA is a simple and well functional wrapped JSON validator
from jsonschema import Draft4Validator, ValidationError
from jsonschema.validators import create
from jsonschema.compat import iteritems
from jsonschema._utils import find_additional_properties

from .exceptions import DTDSchemaError, DTDValidationError, DTDFlexError

VALUEED_TYPES = ('boolean', 'integer', 'null', 'number', 'string')


def _value(validator, value, instance, schema):
    if instance != value:
        yield ValidationError('%s is not the value %s' % (instance, value))


# add `value` schema to validate equality conveniently
draft4_schema_ext = Draft4Validator.META_SCHEMA.copy()
draft4_schema_ext['properties']['value'] = {
    'oneOf': [
        {'type': 'string'},
        {'type': 'number'},
        {'type': 'array'},
    ],
}

draft4_validators_ext = Draft4Validator.VALIDATORS.copy()
draft4_validators_ext['value'] = _value

ExtDraft4Validator = create(draft4_schema_ext, draft4_validators_ext)

default_validator = ExtDraft4Validator({})

# reverse type map from Python to JSON schema
reversed_types = {}
for schema, types in iteritems(ExtDraft4Validator.DEFAULT_TYPES):
    if isinstance(types, (tuple, list)):
        for type_ in types:
            reversed_types[type_] = schema
    else:
        reversed_types[types] = schema

DEFAULT_SCHEMA_NODE = {}


def get_subnode(root, path):
    node = root
    for step in path:
        node = node[step]
    return node


def get_subpath(name, path):
    sub_path = []
    sub_path.extend(path)
    if name is not None:
        sub_path.append(name)
    return sub_path


def json2schema(node, name=None, path=(), check_value=True):
    """Return a strict validation schema for the given JSON node"""
    node_type = type(node)
    if node_type not in reversed_types:
        for type_, schemta in iteritems(reversed_types):
            if isinstance(node, type_):
                reversed_types[node_type] = schemta
                break
        else:
            raise DTDSchemaError(
                "Unexpected type %s at path %s" % (node_type, path),
                instance=node,
                path=path)

    schema = DEFAULT_SCHEMA_NODE.copy()
    schema['type'] = reversed_types[node_type]
    if schema['type'] == 'array':
        length = len(node)
        if length:
            sub_schemas = []
            sub_path = get_subpath(name, path)
            sub_types = set()
            for sub_node in node:
                sub_schema = json2schema(sub_node, path=sub_path, check_value=False)
                sub_types.add(sub_schema['type'])
                sub_schemas.append(sub_schema)
            if not sub_types.intersection(['array', 'object']):
                if check_value:
                    schema['value'] = node
                sub_schemas = [{'type': _type} for _type in sub_types]
            schema['items'] = {'anyOf': sub_schemas}
            schema['maxItems'] = schema['minItems'] = length
        else:
            schema['value'] = node
            schema['minItems'] = 0
            schema['additionalItems'] = False
    elif schema['type'] == 'object':
        sub_items = []
        sub_path = get_subpath(name, path)
        for sub_name, sub_node in iteritems(node):
            sub_items.append((sub_name, json2schema(sub_node, sub_name, sub_path, check_value=check_value)))
        if sub_items:
            properties = {}
            for sub_name, sub_schema in sub_items:
                properties[sub_name] = sub_schema
            schema['properties'] = properties
            schema['required'] = list(properties)
        schema['additionalProperties'] = False
    elif check_value:
        schema['value'] = node
    return schema


def gen_errors(data, schema, valudator=None):
    """Return an errors' generation"""
    valudator = valudator or default_validator
    for err in valudator.iter_errors(data, schema):
        yield DTDValidationError(
                message=err.message,
                validator=err.validator,
                instance=err.instance,
                schema=err.schema,
                path=err.absolute_path,
                schema_path=err.absolute_schema_path,
                cause=err.cause,
            )


def validate(data, schema, validator=None):
    for err in gen_errors(data, schema, validator):
        raise err


def flex_schema_by_err(schema, err):
    """Flex schema according to the DTDValidationError with handlers defined in this module"""
    handler = globals().get('flex_' + err.validator, None)
    if handler is None:
        raise DTDFlexError(
                'Unhandled validator: %s' % err.validator,
                # message=err.message,
                validator=err.validator,
                instance=err.instance,
                schema=err.schema,
                path=err.path,
                schema_path=err.schema_path,
                cause=err.cause,
            )

    schema_node = get_subnode(schema, list(err.schema_path)[:-1])
    handler(schema_node, err)


def flex_schema_by_errs(schema, errs):
    """Flex schema according to the DTDValidationErrors"""
    for err in errs:
        flex_schema_by_err(schema, err)


def flex_type(schema, err):
    if schema['type'] == 'integer':
        schema['type'] = 'number'
    else:
        raise DTDFlexError(
                'Could not flex type: %s' % schema['type'],
                # message=err.message,
                validator=err.validator,
                instance=err.instance,
                schema=err.schema,
                path=err.path,
                schema_path=err.schema_path,
                cause=err.cause,
            )


def flex_required(schema, err):
    required = set(schema['required']).intersection(err.instance)
    schema['required'] = list(required)


def flex_minItems(schema, err):
    del schema['minItems']


def flex_maxItems(schema, err):
    del schema['maxItems']


def flex_value(schema, err):
    del schema['value']


def flex_additionalProperties(schema, err):
    extras = find_additional_properties(err.instance, err.schema)
    for extra in extras:
        schema['properties'][extra] = json2schema(err.instance[extra], extra, err.path)


def flex_anyOf(schema, err):
    sub_schema = json2schema(err.instance, err.path, check_value=False)
    schema['anyOf'].append(sub_schema)
    sub_types = set([item['type'] for item in schema['anyOf']])
    if not sub_types.intersection(['array', 'object']):
        schema['anyOf'] = [{'type': _type} for _type in sub_types]



# encoding: utf-8
from __future__ import absolute_import, unicode_literals

from functools import wraps

from django.db import transaction

from .models import TestCase, TEST_REQUEST_PARAM, TEST_PARAM, TEST_KEY


def testcase(view):
    @wraps(view)
    def wrapper(request, *args, **kwargs):
        if (not request.REQUEST.get(TEST_PARAM, '').upper() == TEST_KEY):
            return view(request, *args, **kwargs)
        elif request.REQUEST.get(TEST_REQUEST_PARAM, '') == '1':
            with transaction.commit_manually():
                try:
                    return view(request, *args, **kwargs)
                finally:
                    transaction.rollback()

        case = TestCase()
        case.load_request(request)

        with transaction.commit_manually():
            try:
                response = view(request, *args, **kwargs)
            finally:
                transaction.rollback()

        case.load_response(response, False)
        case.save()
        return response

    return wrapper



# encoding: utf-8
from __future__ import absolute_import, unicode_literals

from .err_code import ErrCode


class CustomError(ValueError):
    '''客户操作错误，用于在系统流程设计不完善时，处理客户无意提交的错误请求'''
    def __init__(self, errcode=ErrCode.ERR_COMMON_BAD_PARAM, **kwargs):
        self.code = errcode
        self.kwargs = kwargs
        message = self.kwargs.get('message', errcode.message)
        super(CustomError, self).__init__(message)

    def get_res_dict(self):
        return self.code.get_res_dict(**self.kwargs)



# encoding: utf-8
from __future__ import absolute_import, unicode_literals

import re
import string

from django.core.exceptions import ValidationError
from django.core.validators import (
    RegexValidator, BaseValidator,
)

# copy from utility

COUPON_CODE_CHARS = '23456789ABCDEFGHJKLMNPQRSTUVWXYZ'


LST_PLATE_FOLLLOW_CHARS = tuple(string.ascii_uppercase + string.digits)


class ValidateMixin(object):
    '''
    Extend Validator a method 'is_valid'
    '''

    def is_valid(self, value):
        try:
            self(value)
            return True
        except ValidationError:
            return False


class RegexValidatorPlus(ValidateMixin, RegexValidator):
    pass


class ComplexityValidator(ValidateMixin, BaseValidator):
    min_length = 6
    max_length = 16
    min_types = 3

    char_types = (
        'uppercase', 'lowercase', 'letters', 'digits', 'special', 'punctuation'
    )
    code = 'complexity'
    message = {
        'blanks': '不能含有空白字符',
        'min_length': '不能少于%(min_length)d位字符',
        'max_length': '不能多于%(max_length)d位字符',
        'min_types': '需要包含大写字母、小写字母、数字、特殊字符中的%(min_types)d种以上类型',
        'default': '复杂度不能满足要求',
    }

    def __init__(self, **kwargs):
        self.min_types = kwargs.pop('min_types', self.min_types)
        self.min_length = kwargs.pop('min_length', self.min_length)
        self.max_length = kwargs.pop('max_length', self.max_length)

        checking_map = {}
        for char_type in self.char_types:
            checking_map[char_type] = 0

        for k, v in kwargs.items():
            if k not in self.char_types:
                raise TypeError('ComplexityValidator does not support charactor type: %s' % k)
            checking_map[k] = v

        self.checking_map = checking_map

    def __call__(self, value):
        params = self.__dict__.copy()
        if len(value) < self.min_length:
            raise ValidationError(self.message['min_length'], self.code, params)
        if len(value) > self.max_length:
            raise ValidationError(self.message['max_length'], self.code, params)

        checking_map = self.checking_map.copy()
        char_types = set()
        for character in value:
            if character.isupper():
                checking_map['uppercase'] -= 1
                checking_map['letters'] -= 1
                char_types.add('uppercase')
            elif character.islower():
                checking_map['lowercase'] -= 1
                checking_map['letters'] -= 1
                char_types.add('lowercase')
            elif character.isdigit():
                checking_map['digits'] -= 1
                char_types.add('digits')
            elif character in string.punctuation:
                checking_map['punctuation'] -= 1
                checking_map['special'] -= 1
                char_types.add('special')
            elif not character.isspace():
                checking_map['special'] -= 1
                char_types.add('special')
            else:
                raise ValidationError(self.message['blanks'], self.code, params)

        if len(char_types) < self.min_types:
            raise ValidationError(self.message['min_types'], self.code, params)

        for char_type in self.char_types:
            if checking_map[char_type] > 0:
                raise ValidationError(self.message['default'], self.code, params)


# 密码强度
auth_password = ComplexityValidator(min_length=8, max_length=1024, min_types=3)

# 手机号
mobile = RegexValidatorPlus(
    re.compile(r'^1\d{10}$'),
    '手机号格式不正确')

# 中文
chinese = RegexValidatorPlus(
    re.compile(r'^[\u4e00-\u9fa5]*$'),
    '含有非中文字符')

# 身份证号
strict_id_card = RegexValidatorPlus(
    re.compile(r'^(\d{16})|(\d{17}[\dxX])$'),
    '身份证格式不正确')

id_card = RegexValidatorPlus(
    re.compile(r'^(\d{16})|(\d{17}[\da-zA-Z])$'),
    '身份证格式不正确')



# encoding: utf-8
from __future__ import absolute_import, unicode_literals

from collections import OrderedDict

from django.contrib.admin.sites import AlreadyRegistered


class ViewSite(object):
    """
        Site management for view classes
    """
    def __init__(self, name=None, app_name=None):
        self._registry = OrderedDict()
        self.name = name
        self.app_name = app_name

    def __call__(self, *args, **options):
        """Use as a decorator to register view class"""
        if args:
            assert len(args) == 1
            view_class = args[0]
            self.register(view_class)
            return view_class
        else:
            def wrapper(view_class):
                self.register(view_class, **options)
                return view_class
            return wrapper

    def register(self, view_class, **options):
        if options:
            class Meta:
                pass
            for att, val in options.items():
                setattr(Meta, att, val)
            attrs = {
                '__module__': view_class.__module__,
                'Meta': Meta,
                }
            view_class = type(view_class)(
                view_class.__name__, (view_class, ), attrs)

        path = view_class._meta.path.strip('/')
        if path in self._registry:
            raise AlreadyRegistered("path '%s' has been already registered" % (
                path))
        self._registry[path] = view_class

    def get_urls(self):
        from django.conf.urls import url
        urls = []
        for path, view_class in self._registry.items():
            urls.append(url(r'^%s/?$' % path, view_class.as_view(), name=view_class._meta.name))
        return urls

    @property
    def urls(self):
        return self.get_urls(), self.app_name, self.name

    @property
    def urlpatterns(self):
        return self.get_urls()



# encoding: utf-8
from __future__ import absolute_import, unicode_literals


class DTDError(Exception):
    """Base Error of this Module"""


class DTDSchemaError(DTDError):
    """Raise when translating data to DTD failed"""
    def __init__(self, message, instance, path, cause=None):
        self.message = message
        self.instance = instance
        self.path = path
        self.cause = cause


class DTDProcessError(DTDError):
    """Raise when processing data or schema failed"""
    def __init__(self, message, validator, instance, schema, path, schema_path, cause=None):
        self.message = message
        self.validator = validator
        self.instance = instance
        self.schema = schema
        self.path = path
        self.schema_path = schema_path
        self.cause = cause

    def __repr__(self):
        return '<%s: %s>' % (self.__class__.__name__, self.message)

    def __unicode__(self):
        return self.message


class DTDValidationError(DTDProcessError):
    """Raise when validating data failed"""


class DTDFlexError(DTDProcessError):
    """Raise when flexing schema failed"""



#! usr/bin/env python
# encoding: utf-8

from datetime import timedelta
from django.db import models
from django.contrib import admin
from django.contrib.admin.options import TO_FIELD_VAR
from django.contrib.contenttypes.models import ContentType


class AbstractFieldListFilter(admin.FieldListFilter):
    filter_parameter = None
    url_parameter = None

    def get_parameter_name(self, field_path):
        """ Query parameter name for the URL """
        if self.url_parameter:
            return self.url_parameter
        raise NotImplementedError

    def __init__(self, field, request, params, model, model_admin, field_path):
        self.parameter_name = self.get_parameter_name(field_path)
        super(AbstractFieldListFilter, self).__init__(field, request, params, model, model_admin, field_path)

    def has_output(self):
        """ Whether to show filter """
        return True

    def lookups(self, request, model_admin):
        """ Not using lookups """
        return ()

    def choices(self, cl):
        """ Not used, but required by admin_list_filter template tag """
        return ()

    def queryset(self, request, queryset):
        """ Does the actual filtering """
        if self.used_param():
            filter_parameter = self.filter_parameter if self.filter_parameter else self.parameter_name
            return queryset.filter(**{filter_parameter: self.used_param()})

    def expected_parameters(self):
        """
        Returns the list of parameter names that are expected from the
        request's query string and that will be used by this filter.
        """
        return [self.parameter_name]

    def used_param(self):
        """ Value from the query string"""
        return self.used_parameters.get(self.parameter_name, '')


class RelatedAutocompleteFilter(AbstractFieldListFilter):
    template = 'apiview/related_autocomplete.html'
    model = None

    def get_parameter_name(self, field_path):
        if self.url_parameter:
            field_path = self.url_parameter
        # use related field itself to prevent 'join' statement in DB
        return u'{0}__exact'.format(field_path)

    def __init__(self, field, request, params, model, model_admin, field_path):
        super(RelatedAutocompleteFilter, self).__init__(
            field, request, params, model, model_admin, field_path
        )
        field_model = field.model or self.model
        if field_model:
            content_type = ContentType.objects.get_for_model(
                getattr(field_model, field.name).field.remote_field.model, for_concrete_model=False
            )
            self.grappelli_trick = u'/{app_label}/{model_name}/{to_field_var}={to_field}'.format(
                to_field_var=TO_FIELD_VAR,
                app_label=content_type.app_label,
                model_name=content_type.model,
                to_field=getattr(field, 'to_fields', ('id', ))[0]
            )


class SearchFilter(AbstractFieldListFilter):
    template = 'apiview/search.html'

    def get_parameter_name(self, field_path):
        return u'{0}__exact'.format(field_path)


class ISearchFilter(SearchFilter):
    """ Case-insensitive serach filter """

    def get_parameter_name(self, field_path):
        return u'{0}__iexact'.format(field_path)


# compat
SearchFilterC = ISearchFilter


class StartswithFilter(SearchFilter):
    def get_parameter_name(self, field_path):
        return u'{0}__startswith'.format(field_path)


class IStartswithFilter(SearchFilter):
    def get_parameter_name(self, field_path):
        return u'{0}__istartswith'.format(field_path)


class ContainsFilter(SearchFilter):
    def get_parameter_name(self, field_path):
        return u'{0}__contains'.format(field_path)


class IContainsFilter(SearchFilter):
    def get_parameter_name(self, field_path):
        return u'{0}__icontains'.format(field_path)


class DateFieldListFilter(admin.FieldListFilter):
    template = 'apiview/date_filter.html'

    lookup_suffix = ['exact', 'gte', 'lte', 'isnull', 'lt']

    def __init__(self, field, request, params, model, model_admin, field_path):
        super(DateFieldListFilter, self).__init__(
            field, request, params, model, model_admin, field_path
        )
        self.parameter_name = self.field_path

    def expected_parameters(self):
        return [self.patch_param(suffix)
                for suffix in self.lookup_suffix]

    def queryset(self, request, queryset):
        used_parameters = self.used_parameters
        if isinstance(self.field, models.DateTimeField):
            compat_parameters = self.compat_datetimefield()
        else:
            compat_parameters = self.used_parameters
        self.used_parameters = compat_parameters
        queryset = super(DateFieldListFilter, self).queryset(request, queryset)
        self.used_parameters = used_parameters
        return queryset

    def compat_datetimefield(self):
        compat_parameters = {}
        for k, v in self.used_parameters.items():
            if k.endswith('exact'):
                v = self.field.to_python(v)
                compat_parameters[self.patch_param('gte')] = v
                compat_parameters[self.patch_param('lt')] = v + timedelta(days=1)
            elif k.endswith('lte'):
                v = self.field.to_python(v)
                compat_parameters[self.patch_param('lt')] = v + timedelta(days=1)
            else:
                compat_parameters[k] = v
        return compat_parameters

    def patch_param(self, suffix):
        return '%s__%s' % (self.field_path, suffix)

    def __getattribute__(self, attr):
        '''
        extend attributes 'val_%' to get specified parameter
        '''
        if attr.startswith('val_'):
            suffix = attr[4:]
            if suffix in self.lookup_suffix:
                return self.used_parameters.get(self.patch_param(suffix)) or ''
            else:
                raise AttributeError('%s has no attribute "%s"' % (self, attr))
        return super(DateFieldListFilter, self).__getattribute__(attr)

    def choices(self, cl):
        return ()


class RangeFilter(admin.FieldListFilter):
    template = 'apiview/range_filter.html'

    lookup_suffix = ['gte', 'lte', ]

    def __init__(self, field, request, params, model, model_admin, field_path):
        super(RangeFilter, self).__init__(
            field, request, params, model, model_admin, field_path
        )
        self.parameter_name = self.field_path

    def expected_parameters(self):
        return [self.patch_param(suffix) for suffix in self.lookup_suffix]

    def patch_param(self, suffix):
        return '%s__%s' % (self.field_path, suffix)

    def __getattribute__(self, attr):
        '''
        extend attributes 'val_%' to get specified parameter
        '''
        if attr.startswith('val_'):
            suffix = attr[4:]
            if suffix in self.lookup_suffix:
                return self.used_parameters.get(self.patch_param(suffix)) or ''
            else:
                raise AttributeError('%s has no attribute "%s"' % (self, attr))
        return super(RangeFilter, self).__getattribute__(attr)

    def choices(self, cl):
        return ()



# encoding: utf-8
from __future__ import absolute_import, unicode_literals

from django.conf import settings

from .code import Code


CodeDefine = (
    ('SUCCESS',                     0,      '返回成功'),

    ('ERR_UNKOWN',                  -1,     '未知错误'),
    ('ERR_SYS_ERROR',               -2,     '服务异常'),

    ('ERR_COMMON_BAD_PARAM',        -11,    '参数错误'),
    ('ERR_COMMON_BAD_FORMAT',       -12,    '格式错误'),
    ('ERR_COMMON_PERMISSION',       -13,    '权限错误'),
)

ErrCode = Code(CodeDefine + getattr(settings, "ERROR_CODE_DEFINE", ()))



# encoding: utf-8
from __future__ import absolute_import, unicode_literals

from django.conf import settings
from django.utils.deprecation import MiddlewareMixin


class RequestCompatMiddleware(MiddlewareMixin):
    def process_request(self, request):
        # Kill Csrf
        if getattr(settings, "KILL_CSRF", False):
            setattr(request, '_dont_enforce_csrf_checks', True)
        # 兼容body
        ___ = request.body  # NOQA
        request.REQUEST = request.GET.copy()
        request.REQUEST.update(request.POST)



# encoding: utf-8
"""
Declare the metaclass of the views to manage view generation
"""
from __future__ import absolute_import, unicode_literals

import six

from collections import OrderedDict
from six.moves.urllib.parse import urljoin
from django import http
from django import forms
from django.views.generic import View as DjangoView
from django.utils.functional import cached_property
from django.utils.encoding import force_text, force_str

from .utils import split_camel_name
from .param import Param


def _camel2path(name):
    """GenerateURLs => generate/urls"""
    return '/'.join(split_camel_name(name)).lower() + '/'


def _size_len(text):
    return (len(text) + len(text.encode('utf8'))) // 2


def _format(text, size):
    return text + (" " * (size - _size_len(text)))


class ViewOptions(object):
    """View options class

    * name              resolver name of the URL patterns, default is view name
    * path              URL related path generation from parents
    * param_managed     whether manage parameters before dispatching to handler
    * param_fields      parameter's name and field pairs definition,
                        this attribute will generate all parents' param_fields
    * param_dependency  dependency between parameters,
                        format as (name, ((field, [(name, field), ...]), ...))
    * form              form class used to manage parameters
    * decorators        view's decorators generate with the nearest-first logic,
                        this attribute will generate all parents' decorators
    """

    def __init__(self, options=None, parent=None):
        self.view = None
        self.children = []

        self.name = getattr(options, 'name', '')
        self.decorators = list(reversed(getattr(options, 'decorators', ())))

        if parent:
            p_opts = parent._meta
            self.parent = parent
            self.path = getattr(options, 'path', None)
            self.form = getattr(options, 'form', p_opts.form)
            self.param_managed = getattr(options, 'param_managed', p_opts.param_managed)
            self.param_fields = p_opts.param_fields.copy()
            self.param_dependency = p_opts.param_dependency.copy()
            self.param_fields.update(getattr(options, 'param_fields', ()))
            self.param_dependency.update(getattr(options, 'param_dependency', ()))
            self.decorators.extend(p_opts.decorators)
        else:
            self.path = getattr(options, 'path', '/')
            self.parent = None
            self.form = getattr(options, 'form', forms.Form)
            self.param_fields = OrderedDict(getattr(options, 'param_fields', ()))
            self.param_dependency = OrderedDict(getattr(options, 'param_dependency', ()))
            self.param_managed = getattr(options, 'param_managed', True)

    def contribute_to_class(self, cls, name):
        self.view = cls

        if not self.name:
            self.name = self.view.__name__
        if self.path is None:
            parentpath = self.parent._meta.path
            if not parentpath.endswith("/"):
                parentpath += "/"
            self.path = urljoin(parentpath, _camel2path(self.name))
        if self.parent:
            self.parent._meta.children.append(cls)

        form_attrs = dict(self.param_fields)
        form_attrs['__module__'] = cls.__module__
        cls.param_form = type(self.form)(
            force_str(cls.__name__ + 'Form'),
            (self.form, ),
            form_attrs)

        # if cls.__doc__ is None:
        #     cls.__doc__ = ''
        # cls.__doc__ += '\nParameters:\n'
        # cls.__doc__ += self.format_document(with_header=True)

        setattr(cls, name, self)

    @cached_property
    def param_document(self):
        return self.format_document()

    def format_document(self, fields=None, with_header=True):
        if not self.param_fields:
            return ''

        if fields:
            f_attrs = fields
        else:
            f_attrs = ['type_name', 'required', 'omit', 'default', 'help_text', 'field_info']
        fields = ['name'] + f_attrs
        f_lens = [_size_len(field) for field in fields]
        rows = []
        if with_header:
            rows.append(fields)
        for name, field in self.param_fields.items():
            if field.required:
                field_name = '*%s*' % name
            else:
                field_name = '[%s]' % name
            row = [field_name]
            if f_lens[0] < _size_len(field_name):
                f_lens[0] = _size_len(field_name)
            idx = 1
            for att in f_attrs:
                f_val = force_text(getattr(field, att))
                if f_lens[idx] < _size_len(f_val):
                    f_lens[idx] = _size_len(f_val)
                row.append(f_val)
                idx += 1
            rows.append(row)

        doc_list = []
        final_idx = len(rows[0]) - 1
        for row in rows:
            row_doc = []
            for idx, val in enumerate(row):
                if idx == final_idx:
                    row_doc.append(val)
                else:
                    row_doc.append(_format(val, f_lens[idx]))
            doc_list.append(' | '.join(row_doc))

        return '\n'.join(doc_list)

    def decorate_handler(self, handler):
        for decorator in self.decorators:
            handler = decorator(handler)
        return handler


class ViewMetaclass(type):
    """Metaclass of the view classes"""

    def __new__(mcs, name, bases, attrs):
        parents = [b for b in bases if isinstance(b, ViewMetaclass)]
        assert len(parents) <= 1

        __name__ = attrs.pop('__name__', name)

        new_cls = super(ViewMetaclass, mcs).__new__(mcs, name, bases, attrs)

        option_class = getattr(new_cls, 'option_class', ViewOptions)

        if parents:
            opts = option_class(attrs.pop('Meta', None), parents[0])
        else:
            opts = option_class(attrs.pop('Meta', None))

        opts.contribute_to_class(new_cls, '_meta')

        new_cls.__name__ = __name__

        return new_cls


class View(six.with_metaclass(ViewMetaclass, DjangoView)):
    """Base view generated from django view"""

    option_class = ViewOptions

    def dispatch(self, request, *args, **kwargs):
        handler_name = request.method.lower()
        try:
            if handler_name in self.http_method_names:
                # opts = self._meta
                if hasattr(self, handler_name):
                    handler = getattr(self, handler_name)
                else:
                    handler = self.http_method_not_allowed

                request.params = Param(self, request, kwargs)
                response = handler(request, *args, **kwargs)
            else:
                handler = self.http_method_not_allowed
                response = handler(request, *args, **kwargs)

            return response
        except forms.ValidationError as exc:
            response = self.handle_param_errors(exc)

    def handle_param_errors(self, exc):
        if hasattr(exc, 'error_dict'):
            return http.HttpResponseBadRequest(exc.error_dict.as_text())
        else:
            return http.HttpResponseBadRequest(exc)

    @classmethod
    def as_view(cls, *args, **kwargs):
        view = super(View, cls).as_view(*args, **kwargs)
        return cls._meta.decorate_handler(view)



# encoding: utf-8
from __future__ import absolute_import, unicode_literals

import types
import logging

CALLER_KEY = '_caller_'


class ManualCallerLogger(logging.Logger):
    """
    Allow to overwrite LogRecord object's attributes with extra
    """
    def makeRecord(self, name, level, fn, lno, msg, args, exc_info, func=None, extra=None, sinfo=None):
        if extra and CALLER_KEY in extra:
            caller = extra.get(CALLER_KEY)
            if isinstance(caller, types.FunctionType):
                f_code = caller.func_code
                fn = f_code.co_filename
                lno = f_code.co_firstlineno
                func = f_code.co_name
            elif isinstance(caller, types.MethodType):
                f_code = caller.im_func.func_code
                fn = f_code.co_filename
                lno = f_code.co_firstlineno
                if isinstance(caller.im_self, types.ClassType):
                    func = caller.im_self.__name__
                else:
                    func = caller.im_class.__name__

        return super(ManualCallerLogger, self).makeRecord(name, level, fn, lno, msg, args, exc_info, func, extra, sinfo)



# encoding: utf-8
from __future__ import absolute_import, unicode_literals

import six

from django.core.mail.message import EmailMultiAlternatives
from django.core.mail import get_connection
from django import template
from django.template.loader import get_template


def mail(subject, to, body='', attachments=None, headers=None,
         cc=None, bcc=None, from_email=None, connection=None):
    '''
    normal mailer with EmailMessage
    '''
    if isinstance(to, six.string_types):
        to = [to]
    msg = EmailMultiAlternatives(
        subject=subject, body='', from_email=from_email, to=to, bcc=bcc,
        connection=connection, attachments=attachments, headers=headers, cc=cc)
    msg.attach_alternative(body, 'text/html')
    msg.send()


def render_to_mail(subject, to, context, content='', template_name=None, extend_context=True, **kwargs):
    '''
    extend mailer with template and contexts

    `context`:          dict like or Context objects to be rendered, or a list of them
    `content`:          template string content, ignored when `template` was set
    `template_name`:    template name to be load
    `extend_context`:   whether to extend context to match all mails to only if the `context` is a dict like object,
                        default true

    NOTE:
        the length of `context` should be equal with `to`
    '''
    if isinstance(to, six.string_types):
        to = [to]
    if isinstance(context, (dict, template.Context)):
        if extend_context:
            _context = []
            for x in range(len(to)):
                _context.append(context)
            context = _context
        else:
            context = [context]
    assert len(to) == len(context)

    if template_name:
        template_obj = get_template(template_name)
    else:
        template_obj = template.Template(content)

    connection = kwargs.pop('connection', get_connection())
    messages = []
    for _to, _context in zip(to, context):
        if not isinstance(_context, template.Context):
            _context = template.Context(_context)
        _body = template_obj.render(_context)
        msg = EmailMultiAlternatives(subject=subject, to=(_to, ), body='', **kwargs)
        msg.attach_alternative(_body, 'text/html')
        messages.append(msg)

    connection.send_messages(messages)



# encoding: utf-8
from __future__ import absolute_import, unicode_literals

from django.db import DatabaseError
from django.db import models
from django.db.models.base import ModelBase
from django.db.models.manager import EmptyManager
from django.contrib.auth.models import Group, Permission
from django.utils.functional import cached_property
from django.utils.encoding import force_str

from . import cache


class ModelChangeMixin(object):
    '''Monitor the changed attributes'''

    def __setattr__(self, name, value):
        if not name.startswith('_') and name in self.__dict__:
            self._to_change(name, value)
        object.__setattr__(self, name, value)

    def _to_change(self, name, value):
        if name not in self.changed_map:
            self.changed_map[name] = getattr(self, name, None)
        if value == self.changed_map[name]:
            del self.changed_map[name]

    @cached_property
    def changed_map(self):
        return {}

    def get_origin(self, name):
        return self.changed_map.get(name, getattr(self, name))


class ModelFieldChangeMixin(ModelChangeMixin):
    '''Monitor the changed fields

    NOTE: DOT NOT USE `get_origin` to find the origin
    related object when you are not sure it was cached
    because the rel field would not be kept in
    `changed_map` when its data did not load
    from DB after set it to a new object.
    '''

    def __setattr__(self, name, value):
        if not name.startswith('_') and name in self.__dict__:
            if not hasattr(self._meta, '_attname_map'):
                _attname_map = {}
                _name_map = {}
                _auto_now_names = set()
                cache = self._meta._get_fields(reverse=False, include_hidden=True)
                for field in cache:
                    # model fields defined directly and not primary key
                    if isinstance(field, models.Field) and not field.primary_key:
                        _attname_map[field.attname] = field
                        _name_map[field.name] = field
                        if isinstance(field, models.DateField) and field.auto_now:
                            _auto_now_names.add(field.name)

                self._meta._attname_map = _attname_map
                self._meta._name_map = _name_map
                self._meta._auto_now_names = _auto_now_names
            if name in self._meta._attname_map:
                field = self._meta._attname_map.get(name)
                self._to_change(name, value)
                # push only to prevent from these multiple related field
                # such as generic.GenericForeignKey
                self.changed_fields.add(field.name)
            # check the cached related objects into changed map
            elif name in self._meta._name_map:
                field = self._meta._name_map.get(name)
                # ForeignKey or OneToOneField
                if isinstance(field, models.ForeignKey):
                    # whether the data loaded from DB
                    # NOTE: DOT NOT USE `get_origin` to find the origin
                    # related object when you are not sure it was cached
                    # because the rel field would not be kept in
                    # `changed_map` when its data did not load
                    # from DB after set it to a new Model object.
                    if (name in self.__dict__ or getattr(self, field.attname) is None):
                        self._to_change(name, value)
                else:
                    self._to_change(name, value)

        object.__setattr__(self, name, value)

    @cached_property
    def changed_fields(self):
        return set()

    def save_changed(self, using=None):
        '''save the changed fields'''
        if self.changed_fields:
            self.changed_fields.update(getattr(self._meta, '_auto_now_names', set()))
            self.save(force_update=True, update_fields=list(self.changed_fields), using=using)
            self.changed_fields.clear()
            self.changed_map.clear()


class BaseModel(models.Model, ModelFieldChangeMixin):

    _MODEL_WITH_CACHE = True
    _MODEL_CACHE_TTL = 600

    class Meta:
        abstract = True
        ordering = ['-pk', ]

    @classmethod
    def _prepare(cls):
        if 'edit' not in cls._meta.default_permissions:
            cls._meta.default_permissions += ('edit',)
        ModelBase._prepare(cls)

    @classmethod
    def autocomplete_search_fields(cls):

        if hasattr(cls, '_autocomplete_search_fields'):
            return cls._autocomplete_search_fields

        # Apply keyword searches.
        def construct_search(field_name):
            if field_name.startswith('^'):
                return "%s__istartswith" % field_name[1:]
            elif field_name.startswith('='):
                return "%s__iexact" % field_name[1:]
            elif field_name.startswith('@'):
                return "%s__search" % field_name[1:]
            else:
                return "%s__icontains" % field_name
        return ['pk__iexact', ] + [construct_search(str(search_field)) for search_field in cls.search_fields()]

    @classmethod
    def search_fields(cls):
        if hasattr(cls, '_search_fields'):
            return cls._search_fields
        ret = set()
        for field in cls._meta.fields:
            if not field.db_index and not field.unique:
                continue
            if isinstance(field, models.ForeignKey) or isinstance(field, models.ManyToManyField):
                continue
            if isinstance(field, models.CharField):
                ret.add('^%s' % field.name)
            elif isinstance(field, models.IntegerField) and not field.choices:
                ret.add('=%s' % field.name)
        cls._search_fields = ret
        return ret

    @classmethod
    def get_child_field(cls, attr):
        field_names = attr.split('__')
        head_model = cls
        name = ''
        field = None
        for field_name in field_names:
            if head_model is None:
                raise RuntimeError('{} not found {}'.format(cls, attr))
            field = head_model._meta.get_field(field_name)
            head_model = field.remote_field.model if field.remote_field is not None else None
            if name == '':
                name = field.verbose_name
            else:
                name += ' ' + field.verbose_name
        return isinstance(field, models.BooleanField), name

    def __str__(self):
        return force_str('%s%s(%d)' % (self.__class__.__name__, self._meta.verbose_name, self.pk))

    def __unicode__(self):
        return '%s%s(%d)' % (self.__class__.__name__, self._meta.verbose_name, self.pk)

    @classmethod
    def get_obj_by_pk_from_cache(cls, pk):
        if cls._MODEL_WITH_CACHE:
            return cache.ModelPkCache.get(cls, pk)
        else:
            return cls.objects.filter(pk=pk).first()

    @classmethod
    def flush_cache_by_pk(cls, pk):
        if cls._MODEL_WITH_CACHE and pk is not None:
            cache.ModelPkCache.delete(cls, pk)

    @classmethod
    def get_obj_by_unique_key_from_cache(cls, **kwargs):
        assert len(kwargs) == 1
        key, value = list(kwargs.items())[0]
        if cls._MODEL_WITH_CACHE:
            return cache.ModelUniqueFieldCache.get(cls, key, value)
        else:
            return cls.objects.filter(**{key: value}).first()

    @classmethod
    def flush_cache_by_unique_key(cls, **kwargs):
        assert len(kwargs) == 1
        key, value = list(kwargs.items())[0]
        if cls._MODEL_WITH_CACHE and value is not None:
            cache.ModelUniqueFieldCache.delete(cls, key, value)

    def flush_cache(self):
        self.flush_cache_by_pk(self.pk)
        for field in self._meta.fields:
            if field.unique:
                self.flush_cache_by_unique_key(**field.get_filter_kwargs_for_object(self))

    def delete(self, using=None, keep_parents=False):
        self.flush_cache()
        return super(BaseModel, self).delete(using=using, keep_parents=keep_parents)

    def save(self, force_insert=False, force_update=False, using=None,
             update_fields=None):
        self.flush_cache()
        if not (force_insert or force_update or update_fields):
            if self._meta.select_on_save:
                pk_val = self._meta.model.objects.filter(pk=self.pk).values_list('pk', flat=True).first()
                if pk_val is None:
                    force_insert = True
                else:
                    force_update = True

        if update_fields:
            fd = [f.name for f in self._meta.fields]

            for f in update_fields:
                if f not in fd:
                    update_fields.remove(f)
        try:
            super(BaseModel, self).save(force_insert=force_insert,
                                        force_update=force_update,
                                        using=using,
                                        update_fields=update_fields)
        except DatabaseError as exp:
            if str(exp).endswith('did not affect any rows.'):
                pass
            else:
                raise


class AbstractUserMixin(object):
    is_staff = False
    is_active = True
    is_superuser = False
    _groups = EmptyManager(Group)
    _user_permissions = EmptyManager(Permission)

    def _get_groups(self):
        return self._groups
    groups = property(_get_groups)

    def _get_user_permissions(self):
        return self._user_permissions
    user_permissions = property(_get_user_permissions)

    def get_group_permissions(self, obj=None):
        return set()

    def get_all_permissions(self, obj=None):
        return set()

    def has_perm(self, perm, obj=None):
        return False

    def has_perms(self, perm_list, obj=None):
        return False

    def has_module_perms(self, module):
        return False

    def get_username(self):
        return ''



# encoding: utf-8
from __future__ import absolute_import, unicode_literals

import six

from django.db import models
from rest_framework import serializers
from rest_framework.fields import empty

from . import utility


class DictChoiceField(serializers.ChoiceField):

    def __init__(self, choices, **kwargs):
        super(DictChoiceField, self).__init__(choices, **kwargs)
        self.choice_strings_to_values = {
            six.text_type(key): {'key': key, 'desc': self.choices[key]} for key in self.choices.keys()
            }


class DateStampField(serializers.DateField):
    def to_representation(self, value):
        return utility.datetime2timestamp(value)

    def to_internal_value(self, data):
        return utility.timestamp2datetime(int(data)).date()


class DateTimeStampField(serializers.DateTimeField):
    def to_representation(self, value):
        return utility.datetime2timestamp(value)

    def to_internal_value(self, data):
        return utility.timestamp2datetime(int(data))


# class DateFormatField(serializers.DateField):
#     def __init__(self, format=empty, input_formats=None, *args, **kwargs):
#         if format == empty:
#             format = settings.DATE_FORMAT
#         super(DateFormatField, self).__init__(format=format, input_formats=input_formats, *args, **kwargs)
#
#
# class DateTimeFormatField(serializers.DateTimeField):
#     def __init__(self, format=empty, input_formats=None, *args, **kwargs):
#         if format == empty:
#             format = settings.DATETIME_FORMAT
#         super(DateTimeFormatField, self).__init__(format=format, input_formats=input_formats, *args, **kwargs)
#
#
# class TimeFormatField(serializers.TimeField):
#     def __init__(self, format=empty, input_formats=None, *args, **kwargs):
#         if format == empty:
#             format = settings.TIME_FORMAT
#         super(TimeFormatField, self).__init__(format=format, input_formats=input_formats, *args, **kwargs)

#
# class FileField(serializers.FileField):
#     def to_representation(self, value):
#         import pdb;pdb.set_trace()
#         return super(FileField, self).to_representation(value)
#     # def to_internal_value(self, value):
#     #     pass
#
#
# class ImageField(serializers.ImageField):
#     def to_representation(self, value):
#         return super(ImageField, self).to_representation(value)
#     # def to_internal_value(self, value):
#     #     pass
#
#
# class FilePathField(serializers.FilePathField):
#     def to_representation(self, value):
#         pass
#     def to_internal_value(self, value):
#         pass

# class SerializerMetaclass(serializers.SerializerMetaclass):
#     '''处理兼容问题'''
#     def __new__(cls, name, bases, attrs):
#         new_attrs = {}
#         import pdb; pdb.set_trace()
#         for att, val in attrs.items():
#             # 与默认方法名冲突的兼容性问题，
#             # 按照新的Field声明规则，重新声明，保持除method_name外其他参数一致
#             if (isinstance(val, serializers.SerializerMethodField)
#                     and val.method_name == b'get_' + att):
#                 if val._args:
#                     args = val._args[1:]
#                     kwargs = val._kwargs
#                 else:
#                     args = val._args
#                     kwargs = val._kwargs
#                     kwargs.pop('method_name', None)
#                 val = serializers.SerializerMethodField(*args, **kwargs)
#             new_attrs[att] = val
#         return super(SerializerMetaclass, cls).__new__(cls, name, bases, new_attrs)
#

# @six.add_metaclass(SerializerMetaclass)
class BaseSerializer(serializers.ModelSerializer):

    def __init__(self, instance=None, data=empty, **kwargs):
        if data is None:
            data = empty

        request = kwargs.pop("request", None)
        context = kwargs.get("context", {})
        if request is not None:
            context.update({"request": request})

        super(BaseSerializer, self).__init__(instance, data, **kwargs)
        self._context = context
        root = self.root
        if root is not None:
            root._context = context
        fields = self._readable_fields
        for field in fields:
            if isinstance(field, BaseSerializer):
                if not hasattr(field, '_context'):
                    field._context = context

        # self.serializer_field_mapping[models.DateField] = DateFormatField
        # self.serializer_field_mapping[models.DateTimeField] = DateTimeFormatField
        # self.serializer_field_mapping[models.TimeField] = TimeFormatField
        # self.serializer_field_mapping[models.FileField] = FileField
        # self.serializer_field_mapping[models.ImageField] = ImageField
        self.serializer_choice_field = DictChoiceField

    def __new__(cls, *args, **kwargs):
        # We override this method in order to automagically create
        # `ListSerializer` classes instead when `many=True` is set.
        request = kwargs.pop("request", None)
        if "context" not in kwargs and request is not None:
            kwargs["context"] = {"request": request}
        return super(BaseSerializer, cls).__new__(cls, *args, **kwargs)

    @property
    def request(self):
        return self.context.get("request", None)

    def to_representation(self, instance):
        return super(BaseSerializer, self).to_representation(instance)

    @property
    def data(self):
        # 新版data属性使用需要一些前置条件，因此重载使之可以直接使用
        if self.instance is None:
            if hasattr(self, 'many'):
                return []
            else:
                return None
        elif hasattr(self, 'initial_data') and not hasattr(self, '_validated_data'):
            self.is_valid()
        return super(BaseSerializer, self).data

    def build_standard_field(self, field_name, model_field):

        return super(BaseSerializer, self).build_standard_field(field_name, model_field)


class DateStampSerializer(BaseSerializer):
    def __init__(self, instance=None, data=empty, **kwargs):
        super(DateStampSerializer, self).__init__(instance, data, **kwargs)
        self.serializer_field_mapping[models.DateField] = DateStampField


class DateTimeStampSerializer(BaseSerializer):
    def __init__(self, instance=None, data=empty, **kwargs):
        super(DateTimeStampSerializer, self).__init__(instance, data, **kwargs)
        self.serializer_field_mapping[models.DateTimeField] = DateTimeStampField

# class DynamicFieldsModelSerializer(BaseSerializer):
#     """
#     A ModelSerializer that takes an additional `fields` argument that
#     controls which fields should be displayed.
#     """
#
#     def __init__(self, *args, **kwargs):
#         # Don't pass the 'fields' arg up to the superclass
#         fields = kwargs.pop('fields', None)
#
#         # Instantiate the superclass normally
#         super(DynamicFieldsModelSerializer, self).__init__(*args, **kwargs)
#
#         if fields:
#             # Drop any fields that are not specified in the `fields` argument.
#             allowed = set(fields)
#             existing = set(self.fields.keys())
#             for field_name in existing - allowed:
#                 self.fields.pop(field_name)



# encoding: utf-8
from __future__ import absolute_import, unicode_literals

import six
import copy
import mimetypes
from functools import wraps, reduce


from django import forms
from django.contrib import admin, messages
from django.contrib.admin.utils import flatten_fieldsets, unquote
from django.contrib.admin.views import main
from django.contrib.admin.widgets import AdminFileWidget
from django.contrib.auth import get_permission_codename
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import PermissionDenied, ValidationError, FieldDoesNotExist
from django.core.validators import EMPTY_VALUES
from django.db import models, DJANGO_VERSION_PICKLE_KEY
from django.db.models.constants import LOOKUP_SEP
from django.db.models.fields.related import RelatedField
from django.utils.safestring import mark_safe
from django.db.models.query import QuerySet
from django.forms.models import modelform_factory, BaseModelFormSet, BaseInlineFormSet
from django.http import HttpResponseRedirect, Http404
from django.shortcuts import get_object_or_404
from django.template.response import TemplateResponse
from django.urls import reverse, NoReverseMatch
from django.utils.encoding import force_text
from django.utils.http import urlencode
from django.utils.text import capfirst
from django.utils.translation import ugettext as _
from django.utils.version import get_version
from import_export.admin import (
    ExportMixin as _ExportMixin,
    ImportMixin as _ImportMixin,
    ImportExportMixin as _ImportExportMixin,
)
from import_export.resources import ModelDeclarativeMetaclass, ModelResource

from apiview import mailtools, widgets

FIX_COLUMN_VAR = 'fc'
main.IGNORED_PARAMS += (FIX_COLUMN_VAR,)


def set_cache(method, mask, result):
    '''设置方法结果缓存'''
    if hasattr(method, 'im_self'):
        instance = method.im_self
        method_cache = getattr(instance, '_method_cache_', {})
        cache = method_cache.get(method.__name__, {})
        cache['mask'] = mask
        cache['result'] = result
        method_cache[method.__name__] = cache
        instance._method_cache_ = method_cache
    else:
        setattr(method, '_mask_', mask)
        setattr(method, '_result_', result)
        # method._mask_ = mask
        # method._result_ = result


def get_cache(method, mask, default=None):
    '''获取方法结果缓存，在识别码变更时清除缓存'''
    if hasattr(method, 'im_self'):
        instance = method.im_self
        method_cache = getattr(instance, '_method_cache_', {})
        cache = method_cache.get(method.__name__, {})
        if cache.get('mask', None) == mask:
            return cache.get('result', default)
        method_cache.pop(method.__name__, None)
    elif hasattr(method, '_mask_'):
        if method._mask_ == mask:
            return method._result_
        del method._mask_
        del method._result_
    return default


def get_mask(*args):
    return map(id, args)


def get_normal_fields(model):
    fields = []
    for field in model._meta.concrete_fields:
        if not (field.primary_key
                or isinstance(field, RelatedField)
                or isinstance(field, models.FileField)
                or field.name.startswith('_')):
            fields.append(field)
    return fields


class BaseModelResource(ModelResource):
    '''
    Use fields' verbose names as column names
    '''

    @classmethod
    def field_from_django_field(cls, field_name, django_field, readonly):
        field = super(BaseModelResource, cls).field_from_django_field(
            field_name, django_field, readonly
        )
        if field:
            field.column_name = django_field.verbose_name
        return field

    def export(self, queryset=None):

        import tablib
        """
        Exports a resource.
        """
        if queryset is None:
            queryset = self.get_queryset()
        headers = self.get_export_headers()
        data = tablib.Dataset(headers=headers)

        if isinstance(queryset, QuerySet):
            if queryset._prefetch_related_lookups:
                # Could not enjoy the 'prefetch_related' method's benefits when
                # using 'iterator' to generate data
                iterable = iter(queryset)
            else:
                # Iterate without the queryset cache, to avoid wasting memory
                # when exporting large datasets.
                iterable = queryset.iterator()
        else:
            iterable = queryset
        for obj in iterable:
            data.append(self.export_resource(obj))
        return data


def modelresource_factory(model, resource_class=BaseModelResource):
    meta = copy.copy(resource_class._meta)
    meta.model = model
    class_name = model.__name__ + str('Resource')
    class_attrs = {'Meta': meta, }
    return ModelDeclarativeMetaclass(class_name, (resource_class,), class_attrs)


def extend_admincls(*admin_classes):
    assert (admin_classes)
    return type(b'_Admin', admin_classes, {})


def get_widget(field):
    if isinstance(field, models.ImageField):
        return widgets.ImageWidget
    return widgets.TagWidget


def get_fieldwidget(field):
    if isinstance(field, models.ImageField):
        return widgets.ImageFieldWidget
    return widgets.FieldWidget


def _lookup_field(name, obj):
    splices = name.split(LOOKUP_SEP)
    field_name = splices[-1]
    for gen in splices[:-1]:
        obj = getattr(obj, gen)

    field = obj._meta.get_field(field_name)
    if isinstance(field_name, RelatedField):
        label = field.remote_field.field.verbose_name
    else:
        label = field.verbose_name

    return field, label, getattr(obj, field.attname)


def format_field(verbose, field, options=None, **kwargs):
    '''
    Format field with given widgets or default

    Example:
        class MyAdminClass(models.ModelAdmin):
            list_display = [format_field(
                verbose='名字', field='username', options={
                    'style': 'color:red'
                }), ...]
    '''
    name = field
    widget_opts = options or {}

    def _format_field(obj):
        field, label, value = _lookup_field(name, obj)
        attrs = widget_opts.copy()
        widgetclass = attrs.pop('widgetclass', None)
        if widgetclass is None:
            widgetclass = get_widget(field)
        widget = widgetclass(attrs)
        return widget.render(label, value, {})

    _format_field.short_description = verbose
    _format_field.allow_tags = True
    for att, val in kwargs.items():
        setattr(_format_field, att, val)
    return _format_field


def collapse_fields(verbose, fields, options=None, **kwargs):
    '''
    Collapse_fields into one field in changelist_view in modeladmin site

    Example:
        class MyAdminClass(models.ModelAdmin):
            list_display = [collapse_fields(
                verbose='名字', fields=('username', 'name'), options={
                    'username':{'style': 'color:red'}
                }), ...]
    '''
    widget_map = {}
    for name in fields:
        if options:
            widget_kwargs = options.get(name, {})
            widgetclass = widget_kwargs.pop('widgetclass', None)
            widget_map[name] = {
                'widgetclass': widgetclass,
                'attrs': widget_kwargs,
            }
        else:
            widget_map[name] = {}

    def format_fields(obj):
        html = []
        for name in fields:
            field, label, value = _lookup_field(name, obj)
            widgetclass = widget_map[name].get('widgetclass', None) or get_fieldwidget(field)
            widget = widgetclass(widget_map[name].get('attrs', {}))
            html.append(widget.render(name, (label, value), None))
        return ''.join(html)

    format_fields.short_description = verbose
    format_fields.allow_tags = True
    for att, val in kwargs.items():
        setattr(format_fields, att, val)
    return format_fields


def limit_queryset(limits=None, base=QuerySet):
    '''
    return LimitQuerySet to set max rows and count returned by Database
    '''

    class LimitQuerySet(LimitQuerySetMixin, base):
        LIMIT = limits

    return LimitQuerySet


def get_related_model_fields(model, rel, is_foreign_key):
    '''通过model下rel对象获取相关字段或关联属性'''
    # 多对多关联的REL对象本身不区分关系前后
    # 相关代理类做同样处理
    if is_foreign_key and rel.field.model._meta.concrete_model == model._meta.concrete_model:
        return rel.field, rel.get_related_field()
    return rel.get_related_field(), rel.field


class StrictModelFormSet(BaseModelFormSet):
    '''Add validation to ensure POST data is up to date
    '''
    def _existing_object(self, pk):
        if not hasattr(self, '_object_dict'):
            queryset = self.get_queryset()
            if self.data:   # 提交数据时
                pk_field = self.model._meta.pk
                pks = []
                for i in range(self.total_form_count()):
                    prefix = self.add_prefix(i)
                    pk_val = self.data.get('%s-%s' % (prefix, pk_field.name))
                    if pk_val is not None:
                        pks.append(pk_val)
                queryset = queryset.filter(pk__in=pk_val)
            self._object_dict = {o.pk: o for o in queryset}
        obj = self._object_dict.get(pk)
        if obj is None:
            obj = self.get_queryset().filter(pk=pk).first()
            self._object_dict[pk] = obj
        return obj

    def clean(self):
        super(StrictModelFormSet, self).clean()
        self.validate_queryset()

    def validate_queryset(self):
        pk = self.model._meta.pk
        to_python = pk.to_python
        rel = pk.remote_field
        while rel:
            related_field = rel.get_related_field()
            to_python = related_field.to_python
            rel = related_field.remote_field

        updated = False
        for form in self.forms:
            pk_data = form[pk.name].data
            if (pk_data not in EMPTY_VALUES
                    and form.instance.pk != to_python(pk_data)):
                updated = True
                break
        if updated:
            new_formset = self.__class__(
                prefix=self.prefix,
                queryset=self.queryset,
            )
            self.forms = new_formset.forms
            raise ValidationError('页面数据已经更新，请修改后重新保存')


class StrictInlineFormSet(BaseInlineFormSet):
    '''Add validation to ensure POST data is up to date
    '''
    def _existing_object(self, pk):
        if not hasattr(self, '_object_dict'):
            queryset = self.get_queryset()
            if self.data:   # 提交数据时
                pk_field = self.model._meta.pk
                pks = []
                for i in range(self.total_form_count()):
                    prefix = self.add_prefix(i)
                    pk_val = self.data.get('%s-%s' % (prefix, pk_field.name))
                    if pk_val is not None:
                        pks.append(pk_val)
                queryset = queryset.filter(pk__in=pk_val)
            self._object_dict = {o.pk: o for o in queryset}
        obj = self._object_dict.get(pk)
        if obj is None:
            obj = self.get_queryset().filter(pk=pk).first()
            self._object_dict[pk] = obj
        return obj

    def clean(self):
        super(StrictInlineFormSet, self).clean()
        self.validate_queryset()

    def validate_queryset(self):
        pk = self.model._meta.pk
        to_python = pk.to_python
        rel = pk.remote_field
        while rel:
            related_field = rel.get_related_field()
            to_python = related_field.to_python
            rel = related_field.remote_field

        updated = False
        for form in self.forms:
            pk_data = form[pk.name].data
            if (pk_data not in EMPTY_VALUES
                    and form.instance.pk != to_python(pk_data)):
                updated = True
                break
        if updated:
            new_formset = self.__class__(
                save_as_new=self.save_as_new,
                prefix=self.prefix,
                queryset=self.queryset,
            )
            self.forms = new_formset.forms
            raise ValidationError('页面数据已经更新，请修改后重新保存')


class AdminImageWidget(AdminFileWidget):
    def render(self, name, value, attrs=None, renderer=None):
        output = []
        if value and getattr(value, "url", None):
            image_url = value.url
            file_name = str(value)
            output.append(' <a href="%s" target="_blank">'
                          '<img src="%s" alt="%s" style="max-width: 500px; max-height: 1000px" /></a>' %
                          (image_url, image_url, file_name))
        output.append(super(AdminFileWidget, self).render(name, value, attrs, renderer))
        return mark_safe(''.join(output))


# 所有proxy中ModelAdmin的基类
class ProxyModelAdmin(admin.ModelAdmin):
    '''default Admin class used by proxy|real models
    '''

    # db_for_read = DEFAULT_DB_ALIAS
    # db_for_write = DEFAULT_DB_ALIAS

    # remove "__str__"
    list_display = []

    # Extend options to manage site
    # extend field exclude RelatedField and PrimaryKey fields into list_display
    extend_normal_fields = True
    exclude_list_display = []
    heads = []
    tails = []
    # extend LimitQuerySet
    limits = 100
    # manage Add/Change view
    addable = True
    editable = True
    changeable = True
    # manage Change view
    change_view_readonly_fields = []
    editable_fields = None

    # precision
    precision = 2

    # cache key serials set
    cache_serial = set()

    actions = ['_reset']

    formset = StrictModelFormSet

    def __getattr__(self, attr):
        if ('__' in attr
                and not attr.startswith('_')
                and not attr.endswith('_boolean')
                and not attr.endswith('_short_description')):

            def dyn_lookup(instance):
                # traverse all __ lookups
                return reduce(lambda parent, child: getattr(parent, child),
                              attr.split('__'),
                              instance)

            # get admin_order_field, boolean and short_description
            dyn_lookup.admin_order_field = attr
            dyn_lookup.boolean = getattr(self, '{}_boolean'.format(attr), False)
            dyn_lookup.short_description = getattr(
                self, '{}_short_description'.format(attr),
                attr.replace('_', ' ').capitalize())

            return dyn_lookup

        # not dynamic lookup, default behaviour
        return self.__getattribute__(attr)

    def _reset(self, request, querset):
        pass

    _reset.short_description = '清空选项'

    def formfield_for_dbfield(self, db_field, **kwargs):
        if isinstance(db_field, models.ImageField):
            kwargs.pop("request", None)
            kwargs['widget'] = AdminImageWidget
            return db_field.formfield(**kwargs)
        return super(ProxyModelAdmin, self).formfield_for_dbfield(db_field, **kwargs)

    def get_editable_fields(self, request, obj=None):
        if not self.editable:
            return ()
        if self.editable_fields == forms.ALL_FIELDS:
            return None
        elif self.editable_fields is None:
            return ()

        return self.editable_fields

    def get_queryset(self, request):
        '''extend LimitQuerySet to manage rows
        '''
        queryset = super(ProxyModelAdmin, self).get_queryset(request)

        if self.limits:
            klass = limit_queryset(self.limits, queryset.__class__)
            queryset = klass.from_queryset(queryset)

        if not self.has_delete_permission(request) and not self.has_edit_permission(request):
            queryset._for_write = False
            queryset._db = queryset.db
        else:
            queryset._for_write = True
            queryset._db = queryset.db
        return queryset

    def get_model_perms(self, request):

        return {
            'add': self.has_add_perm(request),
            'edit': self.has_edit_perm(request),
            'change': self.has_change_perm(request),
            'delete': self.has_delete_perm(request),
        }

    def has_delete_permission(self, request, obj=None):
        '''add inspection of changeable and editable option
        '''
        if self.editable:
            return super(ProxyModelAdmin, self).has_delete_permission(request, obj)
        return False

    def has_delete_perm(self, request, obj=None):
        '''user permission checking inside the view logic
        '''
        return super(ProxyModelAdmin, self).has_delete_permission(request, obj)

    def has_edit_permission(self, request, obj=None):
        opts = self.opts
        codename = get_permission_codename('edit', opts)
        return request.user.has_perm("%s.%s" % (opts.app_label, codename))

    def has_edit_perm(self, request, obj=None):
        '''user permission checking inside the view logic
        '''
        return self.has_edit_permission(request, obj)

    def has_change_permission(self, request, obj=None):
        '''add inspection of changeable option
        '''
        return self.has_change_perm(request, obj)
        # ret = False
        # if obj:
        #     if self.editable:
        #         ret = self.has_change_perm(request, obj)
        # else:
        #     ret = self.has_change_perm(request, obj)
        # return ret

    def has_change_perm(self, request, obj=None):
        '''user permission checking inside the view logic
        '''
        return super(ProxyModelAdmin, self).has_change_permission(request, obj)

    def has_add_permission(self, request):
        '''add inspection of addable and editable option
        '''
        if self.addable and self.editable:
            return super(ProxyModelAdmin, self).has_add_permission(request)
        return False

    def has_add_perm(self, request):
        '''user permission checking inside the view logic
        '''
        return super(ProxyModelAdmin, self).has_add_permission(request)

    def get_user_queryset(self, queryset):
        return queryset

    def get_form(self, request, obj=None, **kwargs):
        if 'fields' in kwargs:
            fields = kwargs.get('fields')
        else:
            # 'get_fieldsets' would call 'get_form' again with kwargs: field=None
            fields = flatten_fieldsets(self.get_fieldsets(request, obj))

        # get a blank form to fetch all fields
        form = modelform_factory(self.model, kwargs.get('form', self.form), exclude=())

        if fields is None or fields is forms.ALL_FIELDS:
            fields = form.base_fields.keys()

        readonly_fields = self.get_readonly_fields(request, obj, fields=fields)
        readonly_fields_set = set(readonly_fields)

        if 'exclude' not in kwargs:
            if self.exclude is None:
                exclude = []
            else:
                exclude = list(self.exclude)
            exclude.extend(readonly_fields)

            to_exclude_fields = []
            for name in fields:
                field = form.base_fields.get(name, None)
                if field is None:
                    continue
                val = field.prepare_value(getattr(obj, name, None))
                if val is None:
                    continue
                elif name in readonly_fields_set:
                    continue
                elif isinstance(field, forms.ModelMultipleChoiceField):
                    relates = list(val.all().values_list('pk', flat=True))
                    if not relates:
                        continue
                    queryset = self.get_user_queryset(field.queryset)
                    key = field.to_field_name or 'pk'
                    if len(relates) != queryset.filter(**{'%s__in' % key: relates}).count():
                        to_exclude_fields.append(name)
                        readonly_fields_set.add(name)
                elif isinstance(field, forms.ModelChoiceField):
                    queryset = self.get_user_queryset(field.queryset)
                    key = field.to_field_name or 'pk'
                    if not queryset.filter(**{key: val}).exists():
                        to_exclude_fields.append(name)
                        readonly_fields_set.add(name)
                elif isinstance(field, forms.ChoiceField):
                    if field.choices and val not in dict(field.choices):
                        to_exclude_fields.append(name)
                        readonly_fields_set.add(name)

            if to_exclude_fields:
                exclude.extend(to_exclude_fields)
                readonly_fields.extend(to_exclude_fields)
            kwargs['exclude'] = exclude

        kwargs['fields'] = fields
        form = super(ProxyModelAdmin, self).get_form(request, obj, **kwargs)
        # remove the custom fields from base_fields which is in 'exlcude' or not
        # in 'fields' to prevent from checking these fields
        exclude = form._meta.exclude or ()
        for name in form.declared_fields:
            if name in exclude or name not in fields:
                form.base_fields.pop(name, None)
        return form

    def history_view(self, request, object_id, extra_context=None):
        '''
        The 'history' modeladmin view for this model.

        Combine all real models' history into concrete model history view
        '''
        from django.contrib.admin.models import LogEntry
        # First check if the user can see this history.
        model = self.model
        obj = get_object_or_404(model, pk=unquote(object_id))

        if not self.has_change_permission(request, obj):
            raise PermissionDenied

        # Then get the history for this object.
        opts = model._meta
        app_label = opts.app_label
        content_types = [ContentType.objects.get_for_model(model, for_concrete_model=False)]
        for submodel in model.__subclasses__():
            content_types.append(ContentType.objects.get_for_model(submodel, for_concrete_model=False))
        action_list = LogEntry.objects.filter(
            object_id=unquote(object_id),
            content_type__in=content_types,
        ).select_related().order_by('-action_time')

        context = dict(
            self.admin_site.each_context(request),
            title=_('Change history: %s') % force_text(obj),
            action_list=action_list,
            module_name=capfirst(force_text(opts.verbose_name_plural)),
            object=obj,
            app_label=app_label,
            opts=opts,
            preserved_filters=self.get_preserved_filters(request),
        )
        context.update(extra_context or {})

        request.current_app = self.admin_site.name

        return TemplateResponse(request, self.object_history_template or [
            "admin/%s/%s/object_history.html" % (app_label, opts.model_name),
            "admin/%s/object_history.html" % app_label,
            "admin/object_history.html"
        ], context)

    def get_fieldsets(self, request, obj=None):
        fieldsets = list(super(ProxyModelAdmin, self).get_fieldsets(request, obj))
        if not self.has_change_perm(request, obj):
            list_display = self.get_list_display(request)
            valid_f_names = flatten_fieldsets(fieldsets)
            fields = [f for f in list_display if f in valid_f_names]
            return [(None, {'fields': fields})]
        return fieldsets

    def get_readonly_fields(self, request, obj=None, fields=None):
        if obj and self.has_edit_perm(request, obj):

            readonly_fields = list(super(ProxyModelAdmin, self).get_readonly_fields(request, obj))
            readonly_fields_set = set(readonly_fields)
            for field in self.change_view_readonly_fields:
                if field not in readonly_fields_set:
                    readonly_fields_set.add(field)
                    readonly_fields.append(field)

            editable_fields = self.get_editable_fields(request, obj)
            if editable_fields is not None:
                editable_fields_set = set(editable_fields)
                declared_fields = self.fieldsets
                if declared_fields:
                    declared_fields = flatten_fieldsets(declared_fields)
                else:
                    declared_fields = [f.name for f in self.opts.fields]
                for field in declared_fields:
                    if (field not in editable_fields_set
                            and field not in readonly_fields_set):
                        readonly_fields_set.add(field)
                        readonly_fields.append(field)
            return readonly_fields
        elif not self.has_edit_perm(request, obj):
            return fields if fields is not None else self.get_list_display(request)
        else:
            return list(super(ProxyModelAdmin, self).get_readonly_fields(request, obj))

    def get_list_display(self, request):
        '''
        get all fields except PK and Relations if extend_normal_fields is True
        '''

        list_display = list(super(ProxyModelAdmin, self).get_list_display(request))
        if not hasattr(request, '_access_rels'):
            request._access_rels = []

            def _add_access_rels(rel, is_foreign_key):

                target_field, remote_field = get_related_model_fields(self.model, rel, is_foreign_key)
                rel_opts = remote_field.model._meta
                codename = get_permission_codename('change', rel_opts)
                if request.user.has_perm('%s.%s' % (rel_opts.app_label, codename)):
                    try:
                        uri = reverse('admin:%s_%s_changelist' % (
                            rel_opts.app_label, rel_opts.model_name))
                    except NoReverseMatch:
                        pass
                    else:
                        request._access_rels.append((uri, target_field, remote_field, is_foreign_key))

            for rel in self.opts.related_objects:
                #  + tuple(r.remote_field for r in self.opts.many_to_many):
                _add_access_rels(rel, False)
            for key, field in self.opts._forward_fields_map.items():
                if key != field.name:
                    continue
                if field.is_relation and (field.many_to_one or field.one_to_one) \
                        and hasattr(field.remote_field, 'model') and field.remote_field.model:
                    _add_access_rels(field.remote_field, True)

        if request._access_rels:
            self._access_rels = request._access_rels
        else:
            self._access_rels = request._access_rels

        if not self.extend_normal_fields:
            if self._access_rels:
                list_display.append('get_all_relations')
            return list_display

        field_names = []
        heads = []
        middles = []
        tails = []

        def _get_field(field_name):
            if not field_name or not isinstance(field_name, six.string_types):
                return field_name
            try:
                field = self.model._meta.get_field(field_name)
                if field and isinstance(field, models.ImageField):
                    return format_field(field.verbose_name, field.name)
                else:
                    return field_name
            except FieldDoesNotExist:
                return field_name

        while list_display:
            name = list_display.pop(0)
            if name in self.tails:
                tails.append(_get_field(name))
            elif name in self.heads:
                heads.append(_get_field(name))
            else:
                middles.append(_get_field(name))

        for field in self.get_normal_fields():
            if field.name in heads or field.name in middles or field.name in tails:
                continue
            if field.name in self.exclude_list_display:
                pass
            elif field.name in self.tails:
                tails.append(_get_field(field.name))
            elif field.name in self.heads:
                heads.append(_get_field(field.name))
            else:
                middles.append(_get_field(field.name))
        field_names.extend(heads)
        field_names.extend(middles)
        field_names.extend(tails)
        if self._access_rels:
            field_names.append('get_all_relations')
        return field_names

    # def _get_image_field_render(self, field):
    #     def image_tag(obj):
    #         return mark_safe('<img src="%s" style="max-width: 500px; max-height:300px" />'
    #             % (getattr(obj, field.name)))
    #     image_tag.short_description = field.verbose_name
    #     return image_tag

    def get_normal_fields(self):
        fields = []
        exclude = self.exclude or ()
        for field in self.model._meta.concrete_fields:
            if not (isinstance(field, RelatedField)
                    or field.name.startswith('_')
                    or field.attname in exclude):
                if isinstance(field, models.ImageField):
                    # field_name = '_apiview__%s__image_show' % field.name
                    # if not hasattr(self, field_name):
                    #     setattr(self, field_name, self._get_image_field_render(field))
                    # field = field.clone()
                    # field.name = field_name
                    fields.append(field)
                elif not isinstance(field, models.FileField):
                    fields.append(field)
        return fields

    def get_all_relations(self, obj):
        if not self._access_rels:
            return ''
        html_list = ['<select onchange="window.open(this.value, \'_self\');">', ]
        html_list.append('<option value="" selected="selected">------</option>')
        for uri, target_field, remote_field, is_foreign_key in self._access_rels:
            rel_opts = remote_field.model._meta
            value = getattr(obj, target_field.attname)
            value = force_text(value)
            params = {remote_field.name: value}
            url = '%s?%s' % (uri, urlencode(params))
            if is_foreign_key:
                html_list.append('<option value="%s">%s</option>' % (url, target_field.verbose_name))
            else:
                html_list.append('<option value="%s">%s-%s</option>' % (
                    url, rel_opts.verbose_name, remote_field.verbose_name))
        html_list.append('</select>')
        return mark_safe(''.join(html_list))

    get_all_relations.short_description = '相关项'
    get_all_relations.allow_tags = True


def check_perms(*perms):
    """
    Returns True if the given request has permissions to manage an object.
    """

    def inner(func):
        @wraps(func)
        def wrapper(admin, request, *args, **kwargs):
            opts = admin.opts
            for perm in perms:
                codename = get_permission_codename(perm, opts)
                if not request.user.has_perm(
                                "%s.%s" % (opts.app_label, codename)):
                    raise PermissionDenied
            return func(admin, request, *args, **kwargs)

        return wrapper

    return inner


class ImportMixin(_ImportMixin):
    @check_perms('add', 'edit')
    def import_action(self, request, *args, **kwargs):
        return _ImportMixin.import_action(self, request, *args, **kwargs)

    @check_perms('add', 'edit')
    def process_import(self, request, *args, **kwargs):
        return _ImportMixin.process_import(self, request, *args, **kwargs)

    def get_resource_class(self):
        if not self.resource_class:
            return modelresource_factory(self.model, resource_class=BaseModelResource)
        elif self.resource_class._meta.model is None:
            return modelresource_factory(self.model, resource_class=self.resource_class)
        else:
            return self.resource_class


class LimitQuerySetMixin(object):
    LIMIT = None

    def count(self):
        count = super(LimitQuerySetMixin, self).count()
        if self.LIMIT > 0:
            return min(count, self.LIMIT)
        return count

    @classmethod
    def from_queryset(cls, other, **kwargs):
        '''根据其他QuerySet对象的数据生成实例'''
        query = other.query.clone()
        if other._sticky_filter:
            query.filter_is_sticky = True
        clone = cls(model=other.model, query=query, using=other._db, hints=other._hints)
        clone._for_write = other._for_write
        clone._prefetch_related_lookups = other._prefetch_related_lookups[:]
        clone._known_related_objects = other._known_related_objects
        clone._iterable_class = other._iterable_class
        clone._fields = other._fields

        clone.__dict__.update(kwargs)
        return clone

    def iterator(self):
        if self.LIMIT is not None:
            if self.query.low_mark >= self.LIMIT:
                return iter(())
            elif (self.query.high_mark is None
                  or self.query.high_mark > self.LIMIT):
                new_qs = self._clone()
                new_qs.query.high_mark = self.LIMIT
                return new_qs.iterator()
        return super(LimitQuerySetMixin, self).iterator()


class DynQuerySet(LimitQuerySetMixin, QuerySet):
    LIMIT = None

    def __getstate__(self):
        '''pickle adapt, to prevent _fetch_all data'''
        obj_dict = self.__dict__.copy()
        obj_dict[DJANGO_VERSION_PICKLE_KEY] = get_version()
        return obj_dict


def mail_export_data(filename, to, model, resource_class, format_class, queryset):
    '''
    Used to export data
    '''
    if not resource_class._meta.model:
        resource = modelresource_factory(model, resource_class=resource_class)()
    else:
        resource = resource_class()
    file_format = format_class()

    data = resource.export(queryset)
    export_data = file_format.export_data(data)

    content_type = mimetypes.guess_type(filename)[0]
    mailtools.mail(
        '导出结果：' + filename,
        to=to,
        body='请查看附件',
        attachments=[(filename, export_data, content_type)])


class HumanizedModelResource(BaseModelResource):
    '''
    Humanize related fields' value

    Note: this would slow down export processing, even if get related objects
          with 'prefetch_related' to access DB once per field

          to be more efficiency, overwrite 'get_export_date' like this:

            def get_export_data(self, file_format, queryset):
                fields = ["related_field1", ]
                headers = ["related_header1", ]
                for field in self.get_normal_fields():
                    fields.append(field.attname)
                    headers.append(field.verbose_name)

                data = tablib.Dataset(headers=headers)
                for row in queryset.values_list(*fields):
                    data.append(row)
                return file_format.export_data(data)

    '''

    @classmethod
    def field_from_django_field(cls, field_name, django_field, readonly):
        field = super(HumanizedModelResource, cls).field_from_django_field(
            field_name, django_field, readonly
        )
        if field:
            localize_method = 'dehydrate_%s' % field_name
            if django_field.choices:
                setattr(cls, localize_method, lambda self, obj: getattr(obj, 'get_%s_display' % django_field.name)())
            elif isinstance(django_field, RelatedField):
                setattr(cls, localize_method,
                        lambda self, obj: force_text(getattr(obj, django_field.name)))
        return field


class ExportMixin(_ExportMixin):

    def async_export_data(self, func, *args, **kwargs):
        func(*args, **kwargs)

    @check_perms('change')
    def export_action(self, request, *args, **kwargs):
        if request.method == "POST":
            if not request.user.email:
                self.message_user(request, '请设置您的邮箱, 以接收导出数据', messages.ERROR)
            elif not request.POST['file_format']:
                # prompt error
                return _ExportMixin.export_action(self, request, *args, **kwargs)
            else:
                formats = self.get_export_formats()
                try:
                    format_index = int(request.POST['file_format'])
                    file_format = formats[format_index]()
                except Exception:
                    raise Http404

                # from swanleaf.celery.celeryTasks import async_call
                filename = self.get_export_filename(file_format)
                queryset = self.get_export_queryset(request)
                queryset = DynQuerySet.from_queryset(queryset)
                # queryset = queryset.using(BSlave.db_slave1)
                # queryset = queryset.using(DEFAULT_DB_ALIAS)
                if self.resource_class is None:
                    if isinstance(self, ImportMixin):
                        resource_class = BaseModelResource
                    else:
                        resource_class = HumanizedModelResource
                else:
                    resource_class = self.resource_class

                # if (config.SL_EXPORT_DATA_NUM > 0
                #         and queryset.count() > config.SL_EXPORT_DATA_NUM):
                #
                #     if config.SL_EXPORT_DATA_STRICT:
                #         self.message_user(
                #             request,
                #             '导出数据量超出限制(%s条)，请添加筛选条件后重试' % (
                #                 config.SL_EXPORT_DATA_NUM, ),
                #             messages.ERROR)
                #         return HttpResponseRedirect('../?' + request.META['QUERY_STRING'])
                #     else:
                #         self.message_user(
                #             request, '导出数据量过多，处理时间较长，请耐心等待邮件',
                #             messages.WARNING)
                self.async_export_data(mail_export_data, filename, request.user.email, self.model,
                                       resource_class, formats[format_index], queryset)
                # mail_export_data(filename, request.user.email, self.model,
                #                  resource_class, formats[format_index], queryset)
                # async_call(
                #     mail_export_data, filename, request.user.email, self.model,
                #     resource_class, formats[format_index], queryset)

                self.message_user(
                    request, '数据将发送到您的邮箱, 请注意查收',
                    messages.SUCCESS)
            return HttpResponseRedirect('../?' + request.META['QUERY_STRING'])
        return _ExportMixin.export_action(self, request, *args, **kwargs)

    def get_resource_class(self):
        if not self.resource_class:
            return modelresource_factory(self.model, resource_class=HumanizedModelResource)
        elif self.resource_class._meta.model is None:
            return modelresource_factory(self.model, resource_class=self.resource_class)
        else:
            return self.resource_class


class ImportExportMixin(ImportMixin, ExportMixin):
    change_list_template = _ImportExportMixin.change_list_template



# encoding: utf-8
from __future__ import absolute_import, unicode_literals

from django.forms import widgets
from django.utils.translation import ugettext_lazy


class BooleanInput(widgets.CheckboxInput):
    def value_from_datadict(self, data, files, name):
        return data.get(name, '')


class NullBooleanSelect(widgets.Select):
    def __init__(self, attrs=None):
        choices = (
            ('0', ugettext_lazy('Unknown')),
            ('1', ugettext_lazy('yes')),
            ('2', ugettext_lazy('no')),
        )
        super(NullBooleanSelect, self).__init__(attrs, choices)



# encoding: utf-8

from __future__ import absolute_import, unicode_literals

from django.contrib import messages
from django.contrib.admin import ModelAdmin, site
from django.utils.translation import ugettext_lazy as _, ungettext
from django.utils.safestring import mark_safe
from django.utils.encoding import force_text

from .models import TestCase


class TestCaseAdmin(ModelAdmin):
    search_fields = ['path', ]
    list_display = ('request_md5', 'path', 'valid', 'errors')

    readonly = ('path', 'request_md5', 'response_data')

    ordering = ('path', 'id')

    actions = ['delete_selected', 'mark_valid']

    list_per_page = 30

    def mark_valid(self, request, queryset):
        count = 0
        for obj in queryset:
            obj.load_response(obj.test_response, confirm=True)
            obj.save()
            self.log_change(request, obj, 'mark valid')
            count += 1

        self.message_user(request, ungettext(
                'updated %(count)d item',
                'updated %(count)d items',
                count) % {
                    'count': count
            }, messages.SUCCESS)

    mark_valid.short_description = _('mark valid')

    def valid(self, obj):
        try:
            return not obj.errors
        except Exception:
            return False
        return True

    valid.short_description = _('valid')
    valid.boolean = True

    def errors(self, obj):
        error_list = []
        try:
            for err in obj.errors:
                error_list.append('%s: %s' % (' - '.join(map(force_text, err.path)), err.message))
        except Exception as exc:
            error_list.append(force_text(exc))
        return mark_safe('<br/>'.join(error_list))

    errors.short_description = _('errors')


site.register(TestCase, TestCaseAdmin)



# encoding: utf-8
from __future__ import absolute_import, unicode_literals

from django.db import models
from django.core.cache import cache
from django.utils.encoding import force_text


class BaseCacheItem(object):
    _prefix = None
    _expire_secs = 3600 * 24

    @classmethod
    def get_prefix(cls):
        return cls._prefix

    @classmethod
    def _getkey(cls, key):
        assert cls.get_prefix() is not None
        return 'cache:%s:%s' % (cls._prefix, force_text(key))

    @classmethod
    def add(cls, key, value, timeout=None):
        if timeout is None:
            timeout = cls._expire_secs
        return cache.add(cls._getkey(key), value, timeout)

    @classmethod
    def get(cls, key, default=None):
        return cache.get(cls._getkey(key), default)

    @classmethod
    def set(cls, key, value, timeout=None):
        if timeout is None:
            timeout = cls._expire_secs
        return cache.set(cls._getkey(key), value, timeout)

    @classmethod
    def delete(cls, key):
        return cache.delete(cls._getkey(key))

    @classmethod
    def incr(cls, key, delta=1):
        if cls.get(key) is None:
            cls.set(key, 0)
        return cache.incr(cls._getkey(key), delta)

    @classmethod
    def has_key(cls, key):
        return cache.has_key(cls._getkey(key))  # NOQA

    @classmethod
    def ttl(cls, key):
        return cache.ttl(cls._getkey(key))

    @classmethod
    def get_or_set(cls, key, func, timeout=None):
        if timeout is None:
            timeout = cls._expire_secs
        return cache.get_or_set(cls._getkey(key), func, timeout)

    @classmethod
    def persist(cls, key):
        return cache.persist(cls._getkey(key))

    @classmethod
    def expire(cls, key, timeout):
        return cache.expire(cls._getkey(key), timeout)


class AdminFuncCache(BaseCacheItem):
    _prefix = None
    _expire_secs = 3600 * 24


class ModelCacheItem(BaseCacheItem):
    _prefix = 'apiview:model_cache'
    _expire_secs = 600


class ModelUniqueFieldCache(object):

    @classmethod
    def _getkey(cls, modelcls, field_name, field_key):
        assert field_key is not None
        assert issubclass(modelcls, models.Model)
        filed = modelcls._meta.get_field(field_name)
        assert filed.unique
        return "%s:%s:%s" % (modelcls._meta.db_table, filed.name, field_key)

    @classmethod
    def get(cls, modelcls, field_name, field_key):
        ret = ModelCacheItem.get(cls._getkey(modelcls, field_name, field_key))
        if ret is None:
            ret = modelcls.objects.filter(**{field_name: field_key}).first()
            if ret is not None:
                ttl = getattr(modelcls, '_MODEL_CACHE_TTL', None)
                ModelCacheItem.set(cls._getkey(modelcls, field_name, field_key), ret, ttl)
        return ret

    @classmethod
    def delete(cls, modelcls, filed_name, filed_key):
        return ModelCacheItem.delete(cls._getkey(modelcls, filed_name, filed_key))


class ModelPkCache(object):

    @classmethod
    def _getkey(cls, modelcls, pk):
        assert issubclass(modelcls, models.Model)
        return "%s:%s" % (modelcls._meta.db_table, pk)

    @classmethod
    def get(cls, modelcls, pk):
        ret = ModelCacheItem.get(cls._getkey(modelcls, pk))
        if ret is None:
            ret = modelcls.objects.filter(pk=pk).first()
            if ret is not None:
                ttl = getattr(modelcls, '_MODEL_CACHE_TTL', None)
                ModelCacheItem.set(cls._getkey(modelcls, pk), ret, ttl)
        return ret

    @classmethod
    def delete(cls, modelcls, pk):
        return ModelCacheItem.delete(cls._getkey(modelcls, pk))



# encoding: utf-8
'''
HTML Widget classes for display
'''
from __future__ import absolute_import, unicode_literals

import six
from django import forms
from django.forms.utils import flatatt

from django.utils.html import format_html


def styles2python(str_style):
    dt = {}
    for css_item in str_style.strip().split(';'):
        css_item = css_item.strip()
        if not css_item:
            continue
        attr, val = css_item.split(':')
        dt[attr.strip()] = val.strip()
    return dt


def python2styles(dt_style):
    return '; '.join([': '.join(css_pair) for css_pair in six.iteritems(dt_style)])


class TagWidget(forms.Widget):
    tag = 'span'

    css = None

    def __init__(self, *args, **kwargs):
        super(TagWidget, self).__init__(*args, **kwargs)
        self.attrs['css'] = self._patch_inline_style(self.attrs) or {}
        if self.css:
            for att, val in six.iteritems(self.css):
                self.attrs['css'].setdefault(att, val)

    def render(self, name, value, attrs):
        final_attrs = self.build_attrs(attrs)
        self.patch_inline_style(final_attrs)
        return format_html('<{tag} {attrs}>{value}</{tag}>',
                           tag=self.tag,
                           value=value,
                           attrs=flatatt(final_attrs))

    def _patch_inline_style(self, attrs):
        self.tag = attrs.pop('tag', None) or self.tag
        css = attrs.pop('css', None) or self.css
        style = attrs.pop('style', None)
        style = style and styles2python(style)
        if style and css:
            style.update(css)
        elif style or css:
            style = style or css
        else:
            style = None
        return style

    def patch_inline_style(self, attrs):
        style = self._patch_inline_style(attrs)
        if style:
            attrs['style'] = python2styles(style)


class ImageWidget(TagWidget):
    tag = 'img'
    css = {
        'max-width': '80px',
        'max-height': '80px',
    }

    def render(self, name, value, attrs):
        final_attrs = self.build_attrs(attrs)
        self.patch_inline_style(final_attrs)
        if value:
            return format_html('<a target="_blank" href="{url}"><{tag} {attrs} src="{url}" /></a>',
                               tag=self.tag,
                               url=value.url,
                               attrs=flatatt(final_attrs))
        else:
            return ''


class Tag_H4(TagWidget):
    tag = 'h4'
    css = {
        'background': 'transparent',
    }


class Tag_P(TagWidget):
    tag = 'p'


class FieldWidget(forms.MultiWidget):
    name_widget = Tag_H4
    value_widget = Tag_P

    def __init__(self, attrs=None):
        widgets = []
        if attrs is None:
            attrs = {}

        widgets = [
            self.name_widget(attrs),
            self.value_widget(attrs),
            ]
        super(FieldWidget, self).__init__(widgets, attrs)

    def format_output(self, rendered_widgets):
        return ''.join(rendered_widgets)

    def decompress(self, value):
        return value


class ImageFieldWidget(FieldWidget):
    value_widget = ImageWidget

    def format_output(self, rendered_widgets):
        return '<div>{0}</div>'.format(''.join(rendered_widgets))



# encoding: utf-8
"""
View utils package
"""
from __future__ import unicode_literals, absolute_import


def split_camel_name(name, fall=False):
    """Split camel formated names:

    GenerateURLs => [Generate, URLs]
    generateURLsLite => [generate, URLs, Lite]
    """
    if not name:
        return []

    lastest_upper = name[0].isupper()
    idx_list = []
    for idx, char in enumerate(name):
        upper = char.isupper()
        # rising
        if upper and not lastest_upper:
            idx_list.append(idx)
        # falling
        elif fall and not upper and lastest_upper:
            idx_list.append(idx-1)
        lastest_upper = upper

    l_idx = 0
    name_items = []
    for r_idx in idx_list:
        name_items.append(name[l_idx:r_idx])
        l_idx = r_idx
    name_items.append(name[l_idx:])

    return name_items
